package com.yonge.cooleshow.admin.controller;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.yonge.cooleshow.auth.api.client.SysUserFeignService;
import com.yonge.cooleshow.auth.api.entity.SysUser;
import com.yonge.cooleshow.biz.dal.dto.AlbumFavoriteDto;
import com.yonge.cooleshow.biz.dal.entity.AlbumFavorite;
import com.yonge.cooleshow.biz.dal.service.AlbumFavoriteService;
import com.yonge.cooleshow.common.controller.BaseController;
import com.yonge.cooleshow.common.entity.HttpResponseResult;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.validation.Valid;

/**
 * 专辑收藏表 web 控制层
 * @author yzp
 * @date 2022-03-26 00:21:45
 * @version v1.0
 **/
@RestController
@RequestMapping("${app-config.url.admin:}/album/favorite")
@Api(tags = "专辑收藏表 API接口")
public class AlbumFavoriteController extends BaseController {
    @Autowired
    private SysUserFeignService sysUserFeignService;

	@Autowired
	private AlbumFavoriteService albumFavoriteService;

	@ApiOperation(value = "新增", httpMethod="POST", consumes="application/json", produces="application/json")
    @PostMapping(value="/create", consumes="application/json", produces="application/json")
    public HttpResponseResult<Object> create(@Valid @RequestBody AlbumFavorite albumFavorite) {
        SysUser sysUser = sysUserFeignService.queryUserInfo();
        if (sysUser == null) {
            return failed("用户信息获取失败");
        }

        //albumFavorite.setCreateBy(sysUser.getId());
        //albumFavorite.setCreateTime(new Date());
        albumFavoriteService.save(albumFavorite);
        return succeed();
    }

    @ApiOperation(value = "删除", httpMethod="POST", consumes="application/json", produces="application/json")
    @PostMapping("/delete/{id}")
    public Object delete(@PathVariable Long id) {
        SysUser sysUser = sysUserFeignService.queryUserInfo();
        if (sysUser == null) {
            return failed("用户信息获取失败");
        }

        albumFavoriteService.removeById(id);
        return succeed();
    }

    @ApiOperation(value = "修改", httpMethod="POST", consumes="application/json", produces="application/json")
    @PostMapping(value="/update", consumes="application/json", produces="application/json")
    public HttpResponseResult<Object> update(@Valid @RequestBody AlbumFavorite albumFavorite) {
        SysUser sysUser = sysUserFeignService.queryUserInfo();
        if (sysUser == null) {
            return failed("用户信息获取失败");
        }
        if (albumFavorite.getId()==null){
            return failed("缺少ID");
        }

        //albumFavorite.setUpdateBy(sysUser.getId());
        //albumFavorite.setUpdateTime(new Date());
        albumFavoriteService.updateById(albumFavorite);
        return succeed();
    }

    @ApiOperation(value = "分页查询", httpMethod="POST", consumes="application/json", produces="application/json")
    @PostMapping(value="/list", consumes="application/json", produces="application/json")
    public HttpResponseResult<Object> list(@RequestBody AlbumFavoriteDto albumFavoriteDto) {
        SysUser sysUser = sysUserFeignService.queryUserInfo();
        if (sysUser == null) {
            return failed("用户信息获取失败");
        }

        int pageNo = albumFavoriteDto.getPageNo();
        int pageSize = albumFavoriteDto.getPageSize();

        try {
            if (pageNo==0) {
                pageNo = 1;
            }
            if (pageSize==0) {
                pageSize = 10;
            }

            LambdaQueryWrapper<AlbumFavorite> lambdaQueryWrapper = Wrappers.lambdaQuery();
            //lambdaQueryWrapper.like(AlbumFavorite::getName , "k");

            Page<AlbumFavorite> page = new Page<>(pageNo,pageSize);
            return succeed(albumFavoriteService.page(page, lambdaQueryWrapper));
        } catch (Exception e) {
            e.printStackTrace();
            return HttpResponseResult.failed(e.getMessage());
        }
    }
}
