package com.yonge.cooleshow.admin.controller;

import com.yonge.cooleshow.biz.dal.entity.AppVersionInfo;
import com.yonge.cooleshow.biz.dal.queryInfo.AppVersionInfoQueryInfo;
import com.yonge.cooleshow.biz.dal.service.AppVersionInfoService;
import com.yonge.cooleshow.common.controller.BaseController;
import com.yonge.cooleshow.common.entity.HttpResponseResult;
import com.yonge.toolset.base.page.PageInfo;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.MediaType;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.util.List;

@RequestMapping("${app-config.url.admin:}/appVersionInfo")
@Api(tags = "APP版本信息服务")
@RestController
public class AppVersionInfoController extends BaseController {

	@Autowired
	private AppVersionInfoService appVersionInfoService;

	@ApiOperation("分页查询")
	@GetMapping(value = "/list")
	@PreAuthorize("@pcs.hasPermissions('appVersionInfo/list')")
	public HttpResponseResult<PageInfo<AppVersionInfo>> getList(AppVersionInfoQueryInfo queryInfo) {
		return succeed(appVersionInfoService.queryPage(queryInfo));
	}

	@ApiOperation("根据app客户端查询对象")
	@ApiImplicitParam(name = "platform", value = "平台名称", required = true, dataType = "String", paramType = "path")
	@GetMapping(value = "/queryByPlatform")
	public HttpResponseResult<AppVersionInfo> queryByPlatform(String platform,String localVersion) {
		List<AppVersionInfo> list = appVersionInfoService.queryNewestByPlatform(platform);
		if (CollectionUtils.isNotEmpty(list)) {
			AppVersionInfo appVersionInfo = list.get(0);
			//如果应用版本跨多个版本时若其中任何一个版本有强制更新，返回字段标记为强制更新
			if(!appVersionInfo.getIsForceUpdate() && StringUtils.isNotEmpty(localVersion)){
				//是否需要强制更新
				AppVersionInfo forceUpdate = appVersionInfoService.checkForceUpdate(platform,localVersion);
				if(forceUpdate != null){
					appVersionInfo.setIsForceUpdate(true);
				}
			}
			if (StringUtils.isEmpty(localVersion)) {
				appVersionInfo.setIsForceUpdate(true);
			}
			return succeed(appVersionInfo);
		}
		return failed();
	}

	@ApiOperation("单查询")
	@ApiImplicitParam(name = "id", value = "ID编号", required = true, dataType = "Integer", paramType = "path")
	@GetMapping(value = "/query")
	@PreAuthorize("@pcs.hasPermissions('appVersionInfo/query')")
	public HttpResponseResult<AppVersionInfo> query(Long id) {
		return succeed(appVersionInfoService.get(id));
	}

	@ApiOperation("新增")
	@PostMapping(value = "/add", consumes = MediaType.APPLICATION_FORM_URLENCODED_VALUE, produces = MediaType.APPLICATION_JSON_UTF8_VALUE)
	@PreAuthorize("@pcs.hasPermissions('appVersionInfo/add')")
	public Object add(AppVersionInfo appVersionInfo) {
		appVersionInfoService.add(appVersionInfo);
		return succeed();
	}

	@ApiOperation("更新")
	@PostMapping(value = "/update", consumes = MediaType.APPLICATION_FORM_URLENCODED_VALUE, produces = MediaType.APPLICATION_JSON_UTF8_VALUE)
	@PreAuthorize("@pcs.hasPermissions('appVersionInfo/update')")
	public Object update(AppVersionInfo appVersionInfo) {
		appVersionInfoService.updateVersion(appVersionInfo);
		return succeed();
	}

}
