package com.yonge.cooleshow.admin.controller;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.microsvc.toolkit.common.webportal.exception.BizException;
import com.yonge.cooleshow.auth.api.client.SysUserFeignService;
import com.yonge.cooleshow.auth.api.entity.SysUser;
import com.yonge.cooleshow.biz.dal.dao.EmployeeDao;
import com.yonge.cooleshow.biz.dal.dto.EmployeeDto;
import com.yonge.cooleshow.biz.dal.dto.search.EmployeeSearch;
import com.yonge.cooleshow.biz.dal.entity.Employee;
import com.yonge.cooleshow.biz.dal.enums.ClientEnum;
import com.yonge.cooleshow.biz.dal.enums.GenderEnum;
import com.yonge.cooleshow.biz.dal.service.EmployeeService;
import com.yonge.cooleshow.biz.dal.service.im.ImGroupCoreService;
import com.yonge.cooleshow.biz.dal.vo.EmployeeVo;
import com.yonge.cooleshow.biz.dal.wrapper.im.ImGroupWrapper;
import com.yonge.cooleshow.common.controller.BaseController;
import com.yonge.cooleshow.common.entity.HttpResponseResult;
import com.yonge.cooleshow.common.enums.UserLockFlag;
import com.yonge.cooleshow.common.enums.UserStatusEnum;
import com.yonge.cooleshow.common.enums.YesOrNoEnum;
import com.yonge.toolset.base.page.PageInfo;
import com.yonge.toolset.mybatis.support.PageUtil;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import javax.validation.Valid;
import java.util.Date;
import java.util.List;
import java.util.Objects;

@RequestMapping("${app-config.url.admin:}/employee")
@Api(tags = "员工管理")
@RestController
public class EmployeeController extends BaseController {
    @Autowired
    private EmployeeService employeeService;
    @Autowired
    private SysUserFeignService sysUserFeignService;
    @Autowired
    private EmployeeDao employeeDao;

    @Autowired
    private ImGroupCoreService imGroupCoreService;

    @GetMapping("/detail/{id}")
    @ApiOperation(value = "详情", notes = "传入student")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "id", paramType = "path", dataType = "long", required = true),
    })
    @PreAuthorize("@pcs.hasPermissions('employee/detail')")
    public HttpResponseResult<EmployeeVo> detail(@PathVariable("id") Long id) {
        EmployeeVo detail = employeeService.detail(id);
        return succeed(detail);
    }

    /**
     * 查询分页
     */
    @ApiOperation(value = "查询分页", notes = "传入student")
    @PostMapping("/page")
    @PreAuthorize("@pcs.hasPermissions('employee/page')")
    public HttpResponseResult<PageInfo<EmployeeVo>> page(@RequestBody EmployeeSearch query) {
        query.setSort("id_");
    	
    	if(StringUtils.isNotBlank(query.getUserStatus())){
    		switch (query.getUserStatus()) {
			case "LOCKED":
				query.setDelFlag(YesOrNoEnum.NO);
				query.setLockFlag(UserLockFlag.LOCKED);
				break;
			case "CLOSED":
				query.setDelFlag(YesOrNoEnum.YES);
				break;

			default:
				query.setDelFlag(YesOrNoEnum.NO);
				query.setLockFlag(UserLockFlag.NORMAL);
				break;
			}
    	}
        IPage<EmployeeVo> pages = employeeService.selectPage(PageUtil.getPage(query), query);
		List<EmployeeVo> rows = pages.getRecords();
		
		for(EmployeeVo vo : rows){
			if(vo.getDelFlag() == YesOrNoEnum.YES){
				vo.setUserStatus(UserStatusEnum.CLOSED);
			}else{
				if(vo.getLockFlag() == UserLockFlag.LOCKED){
					vo.setUserStatus(UserStatusEnum.LOCKED);
				}else{
					vo.setUserStatus(UserStatusEnum.NORMAL);
				}
			}
		}
        return succeed(PageUtil.pageInfo(pages));
    }

    @ApiOperation(value = "新增员工")
    @PostMapping("/add")
    @PreAuthorize("@pcs.hasPermissions('employee/add')")
    public HttpResponseResult<EmployeeDto> add(@Valid @RequestBody EmployeeDto employee) throws Exception {
        employee.setCreateTime(new Date());
        employee.setUpdateTime(new Date());
        return employeeService.addEmployee(employee);
    }

    @ApiOperation(value = "修改员工")
    @PostMapping("/update")
    @PreAuthorize("@pcs.hasPermissions('employee/update')")
    public HttpResponseResult<EmployeeDto> update(@Valid @RequestBody EmployeeDto employee) throws Exception {
        employee.setUpdateTime(new Date());
        return employeeService.updateEmployee(employee);
    }

    @ApiOperation(value = "获取用户信息")
    @GetMapping("/queryUserInfo")
    public HttpResponseResult<EmployeeVo> apiQueryUserInfo() {
        SysUser sysUser = sysUserFeignService.queryUserInfo();
        if (sysUser != null && sysUser.getId() != null) {
            Employee employee = employeeService.get(sysUser.getId());
            EmployeeVo employeeVo = new EmployeeVo().buildVo(employee);
            employeeVo.setUserId(sysUser.getId());
            employeeVo.setRoleIds(employeeDao.queryUserRole(sysUser.getId()));
            employeeVo.setSysUser(sysUser);

            try {
                ImGroupWrapper.ImUserInfo register = imGroupCoreService.register(
                    employeeVo.getUserId().toString(),
                    ClientEnum.SYSTEM.getCode(),
                    employeeVo.getUsername(),
                    employeeVo.getAvatar()
                );

                // 设置客服标记
                if (Objects.isNull(employeeVo.getCustomerService())) {
                    employeeVo.setCustomerService(false);
                }
                // 获取IMToken
                employeeVo.setImToken(register.getImToken());
                // 获取IM用户ID
                employeeVo.setImUserId(imGroupCoreService.getImUserId(employeeVo.getUserId().toString(), ClientEnum.SYSTEM.getCode()));
            } catch (Exception e) {
                throw new BizException("获取IMToken失败");
            }

            BeanUtils.copyProperties(sysUser, employeeVo, "gender");
            employeeVo.setGender(GenderEnum.valueOf(sysUser.getGender()));
            return succeed(employeeVo);
        }
        return failed("获取用户信息失败");
    }

    @ApiOperation(value = "员工客服绑定关系清除")
    @GetMapping("/clearCustomerService")
    public HttpResponseResult<Boolean> clearCustomerService(@RequestParam("userId") Long userId,
                                                            @RequestParam("sysUserType") String sysUserType) {
        // 清除客服绑定关系
        return succeed(employeeService.clearCustomerService(userId, sysUserType));
    }
}