package com.yonge.cooleshow.admin.controller;

import com.alibaba.fastjson.JSON;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.dayaedu.cbs.common.enums.school.EMusicSheetType;
import com.dayaedu.cbs.openfeign.wrapper.music.CbsMusicSheetWrapper;
import com.microsvc.toolkit.common.response.template.R;
import com.yonge.cooleshow.auth.api.client.SysUserFeignService;
import com.yonge.cooleshow.auth.api.entity.SysUser;
import com.yonge.cooleshow.biz.dal.dao.MusicSheetDao;
import com.yonge.cooleshow.biz.dal.entity.MusicSheet;
import com.yonge.cooleshow.biz.dal.entity.MusicSheetAuthRecord;
import com.yonge.cooleshow.biz.dal.entity.Student;
import com.yonge.cooleshow.biz.dal.enums.ClientEnum;
import com.yonge.cooleshow.biz.dal.enums.SourceTypeEnum;
import com.yonge.cooleshow.biz.dal.service.*;
import com.yonge.cooleshow.biz.dal.vo.MusicSheetDetailVo;
import com.yonge.cooleshow.biz.dal.vo.MusicSheetVo;
import com.yonge.cooleshow.biz.dal.wrapper.InstrumentWrapper;
import com.yonge.cooleshow.biz.dal.wrapper.MusicSheetWrapper;
import com.yonge.cooleshow.common.controller.BaseController;
import com.yonge.cooleshow.common.entity.HttpResponseResult;
import com.yonge.cooleshow.common.enums.YesOrNoEnum;
import com.yonge.toolset.base.exception.BizException;
import com.yonge.toolset.base.page.PageInfo;
import com.yonge.toolset.mybatis.support.PageUtil;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import java.util.Arrays;
import java.util.List;
import java.util.Map;
import java.util.Objects;
import java.util.stream.Collectors;

/**
 * 曲谱表 web 控制层
 * @author yzp
 * @date 2022-03-26 00:21:46
 * @version v1.0
 **/
@RestController
@RequestMapping("${app-config.url.admin:}/musicSheet")
@Api(tags = "曲谱表 API接口")
public class MusicSheetCbsController extends BaseController {
    @Resource
    private SysUserFeignService sysUserFeignService;

	@Resource
	private MusicSheetService musicSheetService;

    @Resource
    private AppVersionInfoService appVersionInfoService;

    @Resource
    private StudentService studentService;

    @Autowired
    private InstrumentService instrumentService;

    @Autowired
    private MusicSheetDao musicSheetDao;

    @Resource
    private MusicSheetAuthRecordService musicSheetAuthRecordService;

    /**
     * 查询单条
     *
     * @param id 详情ID
     * @return R<MusicSheetVo.MusicSheet>
     */
    @ApiOperation(value = "查询内容平台数据")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "id", dataType = "long")
    })
    @GetMapping("/cbsDetailAuth/{id}")
    public R<MusicSheetVo.MusicSheetDetail> cbsDetailAuth(@PathVariable("id") String id,Boolean simpleFlag) {

        MusicSheetVo.MusicSheetDetail musicSheetDetail = null;
        MusicSheetDetailVo detail = musicSheetDao.detail(Long.parseLong(id));
        if (detail == null) {
            MusicSheetDetailVo result = musicSheetAuthRecordService.auditDetail(id);
            if (result != null) {
                detail = result;
                if(StringUtils.isNotEmpty(result.getMusicSheetJson())){
                    // 审核中数据
                    musicSheetDetail = JSON.parseObject(result.getMusicSheetJson(), MusicSheetVo.MusicSheetDetail.class);
                }else {
                    MusicSheetAuthRecord authRecord = musicSheetAuthRecordService.getById(id);
                    detail = musicSheetDao.detail(authRecord.getMusicSheetId());
                    if(detail == null){
                        throw new BizException("未找到曲目信息");
                    }
                    id = authRecord.getMusicSheetId().toString();
                }
            }
            if(result == null){
                throw new BizException("未找到曲目信息");
            }

        }

        if (musicSheetDetail == null) {
            detail = musicSheetService.getCbsDetail(Long.parseLong(id),SourceTypeEnum.PLATFORM);

            CbsMusicSheetWrapper.MusicSheet cbsMusicSheet = musicSheetService.cbsDetail(detail.getCbsMusicSheetId(), simpleFlag);
            cbsMusicSheet.setBizId(Long.parseLong(id));
            musicSheetDetail = JSON.parseObject(JSON.toJSONString(cbsMusicSheet), MusicSheetVo.MusicSheetDetail.class);
        }

        // 设置曲目付费类型
        musicSheetDetail.setPaymentType(detail.getPaymentType());
        // 设置业务端曲目分类
        musicSheetDetail.setScoreType(detail.getScoreType());
        musicSheetDetail.setIsConvertibleScore(detail.getNotation() ==YesOrNoEnum.YES);
        musicSheetDetail.setPlay(detail.getPlay());
        musicSheetDetail.setBuyed(detail.getBuyed());
        musicSheetDetail.setMusicPrice(detail.getMusicPrice());


    // 如果是合奏 并且乐器ID = 2268
        musicSheetDetail.setSpecialPercussionFlag(false);
        if ("2268".equals(musicSheetDetail.getMusicalInstrumentIds()) && musicSheetDetail.getMusicSheetType() == EMusicSheetType.CONCERT) {
            musicSheetDetail.setSpecialPercussionFlag(true);
        }

        // 设置乐器信息
        if (StringUtils.isNotBlank(musicSheetDetail.getMusicalInstrumentIds())) {
            List<Long> instrumentIds = Arrays.stream(musicSheetDetail.getMusicalInstrumentIds().split(","))
                    .map(Long::parseLong).collect(Collectors.toList());

            if (CollectionUtils.isNotEmpty(instrumentIds) && CollectionUtils.isNotEmpty(musicSheetDetail.getMusicalInstruments())) {
                Map<Long, InstrumentWrapper.Instrument> instrumentMap = instrumentService.getMapByIds(instrumentIds);
                for (CbsMusicSheetWrapper.MusicalInstrument musicalInstrument : musicSheetDetail.getMusicalInstruments()) {
                    InstrumentWrapper.Instrument instrument = instrumentMap.get(musicalInstrument.getId().longValue());
                    if (instrument != null) {
                        musicalInstrument.setOrientation(instrument.getOrientation());
                    }

                }
            }
        }
        return R.from(musicSheetDetail);

    }


        /**
         * 查询单条
         *
         * @param id 详情ID
         * @return R<MusicSheetVo.MusicSheet>
         */
    @ApiOperation(value = "查询内容平台数据")
    @ApiImplicitParams({
        @ApiImplicitParam(name = "id", value = "id", dataType = "long")
    })
    @GetMapping("/cbsDetail/{id}")
    public R<MusicSheetVo.MusicSheetDetail> cbsDetail(@PathVariable("id") Long id,Boolean simpleFlag,
                                                      @RequestParam(required = false) String tenantAlbumId,
                                                      @RequestParam(required = false) String providerType) {

        SysUser sysUser = sysUserFeignService.queryUserInfo();
        if (sysUser == null  || sysUser.getId() == null) {
            sysUser = null;
        }
        // 曲目信息
        MusicSheet musicSheet = musicSheetService.getDao().get(id);
        if (Objects.isNull(musicSheet)) {
            throw new BizException("曲目信息不存在");
        }
        if (musicSheet.getCbsMusicSheetId() ==null) {
            throw new BizException("曲目信息异常");
        }
        CbsMusicSheetWrapper.MusicSheet cbsMusicSheet = musicSheetService.cbsDetail(musicSheet.getCbsMusicSheetId(), simpleFlag);
        cbsMusicSheet.setBizId(id);

        // 设置查询机构 还是平台数据
        if (StringUtils.isBlank(tenantAlbumId) && StringUtils.isNotBlank(providerType)) {
            if (SourceTypeEnum.PLATFORM.name().equals(providerType)) {
                tenantAlbumId = null;
            } else {
                tenantAlbumId = "1";
            }
        }

        MusicSheetVo.MusicSheetDetail musicSheetDetail = JSON.parseObject(JSON.toJSONString(cbsMusicSheet), MusicSheetVo.MusicSheetDetail.class);
        MusicSheetDetailVo detail = musicSheetService.detail(id.toString(), sysUser, ClientEnum.SYSTEM, tenantAlbumId);
        if (detail == null) {
        } else {
            // 设置曲目付费类型
            musicSheetDetail.setPaymentType(detail.getPaymentType());
            // 设置业务端曲目分类
            musicSheetDetail.setScoreType(detail.getScoreType());
            musicSheetDetail.setIsConvertibleScore(detail.getNotation() ==YesOrNoEnum.YES);
            musicSheetDetail.setPlay(detail.getPlay());
            musicSheetDetail.setBuyed(detail.getBuyed());
            musicSheetDetail.setMusicPrice(detail.getMusicPrice());
        }

        // 如果是合奏 并且乐器ID = 2268
        musicSheetDetail.setSpecialPercussionFlag(false);
        if ("2268".equals(musicSheetDetail.getMusicalInstrumentIds()) && musicSheetDetail.getMusicSheetType() == EMusicSheetType.CONCERT) {
            musicSheetDetail.setSpecialPercussionFlag(true);
        }

        // 设置乐器信息
        if (StringUtils.isNotBlank(musicSheetDetail.getMusicalInstrumentIds())) {
            List<Long> instrumentIds = Arrays.stream(musicSheetDetail.getMusicalInstrumentIds().split(","))
                .map(Long::parseLong).collect(Collectors.toList());

            if (CollectionUtils.isNotEmpty(instrumentIds) && CollectionUtils.isNotEmpty(musicSheetDetail.getMusicalInstruments())) {
                Map<Long, InstrumentWrapper.Instrument> instrumentMap = instrumentService.getMapByIds(instrumentIds);
                for (CbsMusicSheetWrapper.MusicalInstrument musicalInstrument : musicSheetDetail.getMusicalInstruments()) {
                    InstrumentWrapper.Instrument instrument = instrumentMap.get(musicalInstrument.getId().longValue());
                    if (instrument != null) {
                        musicalInstrument.setOrientation(instrument.getOrientation());
                    }

                }
            }
        }
        return R.from(musicSheetDetail);
    }
}
