package com.yonge.cooleshow.config;

import com.yonge.cooleshow.common.security.BaseAccessDeniedHandler;
import com.yonge.cooleshow.common.security.BaseAuthenticationEntryPoint;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Configuration;
import org.springframework.security.config.annotation.method.configuration.EnableGlobalMethodSecurity;
import org.springframework.security.config.annotation.web.builders.HttpSecurity;
import org.springframework.security.oauth2.config.annotation.web.configuration.EnableResourceServer;
import org.springframework.security.oauth2.config.annotation.web.configuration.ResourceServerConfigurerAdapter;
import org.springframework.security.oauth2.config.annotation.web.configurers.ResourceServerSecurityConfigurer;

@Configuration
@EnableResourceServer
@EnableGlobalMethodSecurity(prePostEnabled = true)
public class ResourceServerConfig extends ResourceServerConfigurerAdapter {

	@Autowired
	private BaseAccessDeniedHandler baseAccessDeniedHandler;

	@Autowired
	private BaseAuthenticationEntryPoint baseAuthenticationEntryPoint;

	@Override
	public void configure(HttpSecurity http) throws Exception {
		http.csrf().disable()
				.exceptionHandling()
				.accessDeniedHandler(baseAccessDeniedHandler)
				.authenticationEntryPoint(baseAuthenticationEntryPoint).and()
				// .authorizeRequests().antMatchers("/task/**").hasIpAddress("0.0.0.0/0")
				.authorizeRequests()
				.antMatchers("/*/task/**").permitAll()
				.antMatchers(
						"/v2/api-docs",
						"/*/v2/api-docs",
						"/*/code/*",
						"/*/api/*",
						"/*/appVersionInfo/queryByPlatform",
						"/*/uploadFile",
						"/*/open/**",
						"/open/**",
						"/*/room/statusSync",
						"/*/room/tencentRtcCallback",
						"/*/wechat/*",
						"/*/sysMusicRecord/getLastEvaluationMusicalNotesPlayStats",
						"/*/sysConfig/queryByParamNameList",
						"/*/music/sheet/detail/**",
						"/*/liveRoom/test",
						"/*/liveRoom/syncUserStatus",
						"/*/liveRoomVideo/recordSync",
						"/*/courseGroup/getLockCache"
				).permitAll()
				.anyRequest().authenticated()
				.and().httpBasic();
	}

	@Override
	public void configure(ResourceServerSecurityConfigurer resources) throws Exception {
		resources.authenticationEntryPoint(baseAuthenticationEntryPoint).accessDeniedHandler(baseAccessDeniedHandler);
	}

}
