package com.yonge.cooleshow.student.controller;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.yonge.cooleshow.biz.dal.dto.CourseHomeworkSubmitDto;
import com.yonge.cooleshow.biz.dal.dto.search.HomeworkSearch;
import com.yonge.cooleshow.biz.dal.entity.StudentAttendance;
import com.yonge.cooleshow.biz.dal.entity.TeacherAttendance;
import com.yonge.cooleshow.biz.dal.enums.ClientEnum;
import com.yonge.cooleshow.biz.dal.enums.CourseScheduleEnum;
import com.yonge.cooleshow.biz.dal.service.*;
import com.yonge.cooleshow.biz.dal.vo.CourseHomeworkDetailVo;
import com.yonge.cooleshow.biz.dal.vo.CourseHomeworkVo;
import com.yonge.cooleshow.common.controller.BaseController;
import com.yonge.cooleshow.common.entity.HttpResponseResult;
import com.yonge.cooleshow.common.enums.YesOrNoEnum;
import com.yonge.toolset.base.page.PageInfo;
import com.yonge.toolset.mybatis.support.PageUtil;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.apache.commons.collections.CollectionUtils;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import javax.validation.Valid;
import java.util.*;
import java.util.stream.Collectors;

/**
 * Description 学生课后作业相关接口
 *
 * @author liujunchi
 * @date 2022-04-13
 */
@Api(tags = "学生课后作业相关接口")
@RestController
@RequestMapping("${app-config.url.student:}/homework")
public class CourseHomeworkController extends BaseController {

    @Resource
    private SysUserService sysUserService;

    @Resource
    private CourseScheduleService courseScheduleService;

    @Resource
    private CourseHomeworkService courseHomeworkService;

    @Resource
    private ImGroupService imGroupService;
    @Resource
    private StudentAttendanceService studentAttendanceService;


    @ApiOperation(value = "首页-我的课程-课程详情(趣纠课)-课后作业信息详情",notes = "传入课程编号ID")
    @GetMapping(value = "/detail/{courseId}")
    public HttpResponseResult<CourseHomeworkDetailVo> detail(@ApiParam(value = "课程编号ID", required = true)
                                                                 @PathVariable("courseId") Long courseId) {

        Long userId = sysUserService.getUserId();
        HttpResponseResult info = checkCourseSchedule(courseId,userId);
        if (info != null) return info;
        CourseHomeworkDetailVo detailVo = courseHomeworkService.getCourseHomeworkDetailByCourseId(courseId, userId);
        if (detailVo != null) {
            detailVo.setImUserId(imGroupService.getImUserId(String.valueOf(detailVo.getTeacherId()),ClientEnum.TEACHER.name()));
            //获取学生考勤信息
            StudentAttendance one = studentAttendanceService.lambdaQuery().eq(StudentAttendance::getCourseScheduleId, courseId)
                    .eq(StudentAttendance::getStudentId, userId).last("limit 1").one();
            detailVo.setAttendanceStatus(Objects.nonNull(one));
        }
        return succeed(detailVo);
    }

    private HttpResponseResult checkCourseSchedule(Long courseId,Long userId) {
        if (!courseScheduleService.checkStudentCourseSchedule(userId, courseId)) {
            return failed("学生只能看自己购买的课程详情");
        }
        return null;
    }


    @ApiOperation(value = "趣纠课-作业提交", httpMethod = "POST", consumes = "application/json", produces = "application/json")
    @PostMapping(value = "/submit", consumes = "application/json", produces = "application/json")
    public HttpResponseResult<Boolean> reviewCourseHomework(@Valid @RequestBody CourseHomeworkSubmitDto submitDto) {
        Long userId = sysUserService.getUserId();
        HttpResponseResult info = checkCourseSchedule(submitDto.getCourseScheduleId(),userId);
        if (info != null) return info;
        submitDto.setStudentId(userId);
        return succeed(courseHomeworkService.submitCourseHomework(submitDto));
    }

    @ApiOperation(value = "课后作业-列表", httpMethod = "POST", consumes = "application/json", produces = "application/json")
    @PostMapping(value = "/list", consumes = "application/json", produces = "application/json")
    public HttpResponseResult<PageInfo<CourseHomeworkVo>> list(@Valid @RequestBody HomeworkSearch query) {
        Long userId = sysUserService.getUserId();
        query.setDecorate(YesOrNoEnum.YES);
        query.setStudentId(userId);
        query.setCourseStatus(CourseScheduleEnum.COMPLETE);
        List<CourseScheduleEnum> list = new ArrayList<>();
        list.add(CourseScheduleEnum.PIANO_ROOM_CLASS);
        list.add(CourseScheduleEnum.PRACTICE);
        list.add(CourseScheduleEnum.VIP);
        list.add(CourseScheduleEnum.GROUP);
        query.setCourseType(list);

        IPage<CourseHomeworkVo> page = courseHomeworkService.selectPage(PageUtil.getPage(query), query);
        List<CourseHomeworkVo> records = page.getRecords();
        if (CollectionUtils.isNotEmpty(records)) {
            for (CourseHomeworkVo item : records) {
                item.setImUserId(imGroupService.getImUserId(String.valueOf(item.getTeacherId()),ClientEnum.TEACHER.name()));
            }
        }

        return succeed(PageUtil.pageInfo(page));
    }


}
