package com.yonge.cooleshow.student.controller;


import com.alibaba.fastjson.JSON;
import com.yonge.cooleshow.biz.dal.entity.ImGroupMember;
import com.yonge.cooleshow.biz.dal.enums.MK;
import com.yonge.cooleshow.biz.dal.service.ImGroupMemberService;
import com.yonge.cooleshow.biz.dal.vo.im.ImUserFriendVO;
import com.yonge.cooleshow.common.controller.BaseController;
import com.yonge.cooleshow.common.entity.HttpResponseResult;
import com.yonge.toolset.base.exception.BizException;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.collections.MapUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.util.List;
import java.util.Map;
import java.util.Objects;

/**
 * 即时通讯群组(ImGroupMember)表控制层
 *
 * @author zx
 * @since 2022-03-22 10:45:56
 */
@Api(tags = "即时通讯群组联系人")
@RestController
@RequestMapping("${app-config.url.student:}/imGroupMember")
public class ImGroupMemberController extends BaseController {
    /**
     * 服务对象
     */
    @Autowired
    private ImGroupMemberService imGroupMemberService;

    @ApiImplicitParams({
            @ApiImplicitParam(name = "groupId", dataType = "Long", value = "群编号",required = true),
            @ApiImplicitParam(name = "userId", dataType = "Long", value = "用户编号",required = true),
    })
    @ApiOperation("获取好友详情")
    @PostMapping(value = "/getUserDetail")
    public HttpResponseResult<ImUserFriendVO.ImGroupMember> getUserDetail(@RequestBody Map<String,Object> params) {
        if(Objects.isNull(params.get("groupId")) || Objects.isNull(params.get("userId"))){
            throw new BizException("参数校验失败");
        }

        String userId = MapUtils.getString(params, "userId", "");

        String ret = userId.split(":")[0];
        if (!ret.matches(MK.EXP_INT)) {
            return failed("无效的用户ID");
        }

        ImGroupMember groupMember = imGroupMemberService.getUserDetail(userId, params.get("groupId").toString());
        if (Objects.isNull(groupMember)) {
            return failed("用户无当前群组权限");
        }

        return succeed(ImUserFriendVO.ImGroupMember.from(JSON.toJSONString(groupMember)));
    }


    @ApiImplicitParams({
            @ApiImplicitParam(name = "search", dataType = "String", value = "根据用户编号、昵称模糊查询"),
            @ApiImplicitParam(name = "groupId", dataType = "Long", value = "群编号")
    })
    @ApiOperation("获取群成员列表")
    @PostMapping(value = "/queryAll")
    public HttpResponseResult<List<ImGroupMember>> queryAll(@RequestBody Map<String,Object> params) throws Exception {
        if(Objects.isNull(params.get("groupId"))){
            throw new BizException("参数校验失败");
        }

        // 群组成员信息
        List<ImGroupMember> groupMembers = imGroupMemberService.findChatGroupAllMemberInfo(params);

        /*Object search = params.get("search");
        List<ImGroupMember> iPage = imGroupMemberService.getBaseMapper().selectList(Wrappers.<ImGroupMember>query().lambda()
                .and(Objects.nonNull(search) && StringUtils.isNotEmpty(search.toString()),
                        e->e.eq(ImGroupMember::getUserId, search).or()
                        .like(ImGroupMember::getNickname, search))
                .eq(ImGroupMember::getGroupId,params.get("groupId")).orderByDesc(ImGroupMember::getId));*/
        return succeed(groupMembers);
    }
}

