package com.yonge.cooleshow.student.controller;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.yonge.cooleshow.biz.dal.dto.CheckCourseTimeDto;
import com.yonge.cooleshow.biz.dal.entity.CourseTimeEntity;
import com.yonge.cooleshow.biz.dal.entity.Subject;
import com.yonge.cooleshow.biz.dal.service.AppVersionInfoService;
import com.yonge.cooleshow.biz.dal.service.CourseGroupService;
import com.yonge.cooleshow.biz.dal.service.SysUserService;
import com.yonge.cooleshow.biz.dal.vo.CourseGroupVo;
import com.yonge.cooleshow.biz.dal.vo.CourseInfoVo;
import com.yonge.cooleshow.biz.dal.vo.LiveCourseInfoVo;
import com.yonge.cooleshow.biz.dal.wrapper.course.CourseGroupWrapper;
import com.yonge.cooleshow.common.controller.BaseController;
import com.yonge.cooleshow.common.entity.HttpResponseResult;
import com.yonge.cooleshow.common.enums.YesOrNoEnum;
import com.yonge.toolset.base.page.PageInfo;
import com.yonge.toolset.mybatis.support.PageUtil;
import io.swagger.annotations.*;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import javax.validation.Valid;
import java.util.List;
import java.util.Map;

/**
 * 课程组表(CourseGroup)表控制层
 *
 * @author hgw
 * @since 2022-03-18 15:29:10
 */
@Api(tags = "课程组表")
@RestController
@RequestMapping("${app-config.url.student:}/courseGroup")
public class StudentCourseGroupController extends BaseController {
    @Resource
    private SysUserService sysUserService;
    @Resource
    private CourseGroupService courseGroupService;
    @Resource
    private AppVersionInfoService appVersionInfoService;

    @ApiOperation("直播课详情")
    @GetMapping("/queryLiveCourseInfo")
    public HttpResponseResult<LiveCourseInfoVo> queryLiveCourseInfo(@ApiParam(value = "课程组id", required = true) @RequestParam(value = "groupId") Long groupId) {
        return succeed(courseGroupService.queryLiveCourseInfo(groupId));
    }

    @ApiOperation("课程组详情")
    @GetMapping("/queryCourseInfo")
    public HttpResponseResult<CourseInfoVo> queryCourseInfo(@ApiParam(value = "课程组id", required = true) @RequestParam(value = "groupId") Long groupId) {
        return succeed(courseGroupService.queryCourseInfo(groupId));
    }

    @ApiImplicitParams({
            @ApiImplicitParam(name = "teacherId", dataType = "Long", value = "老师id"),
            @ApiImplicitParam(name = "search", dataType = "String", value = "关键字"),
            @ApiImplicitParam(name = "groupStatus", dataType = "String", value = "课程组状态  ING(进行中)  NOT_SALE(未开售,未上架) APPLY(报名中,销售中) COMPLETE(已完成)"),
            @ApiImplicitParam(name = "page", dataType = "Integer", value = "页数"),
            @ApiImplicitParam(name = "subjectId", dataType = "Integer", value = "声部id"),
            @ApiImplicitParam(name = "courseType", dataType = "String", value = "课程类型"),
            @ApiImplicitParam(name = "rows", dataType = "Integer", value = "每页数量"),
            @ApiImplicitParam(name = "version", dataType = "String", value = "版本"),
            @ApiImplicitParam(name = "platform", dataType = "Integer", value = "平台"),
    })
    @ApiOperation("分页查询直播课课程组列表")
    @PostMapping("/queryPageCourseGroup")
    public HttpResponseResult<PageInfo<CourseGroupVo>> queryPageLiveCourseGroup(@RequestBody Map<String, Object> param) {
        param.put("os","student");
        YesOrNoEnum auditVersion = YesOrNoEnum.NO;
        if (param.get("platform") != null && param.get("version") != null) {
            auditVersion = appVersionInfoService.getAppAuditVersion(param.get("platform").toString(),param.get("version").toString());
        }
        param.put("auditVersion",auditVersion);
        return succeed(courseGroupService.queryPageLiveCourseGroup(param));
    }

    @ApiOperation("创建直播课程组-锁定课程时间-将课时写到缓存当作锁定的时间")
    @PostMapping("/lockCourseToCache")
    public HttpResponseResult<List<CourseTimeEntity>> lockCourseToCache(@RequestBody @Valid CheckCourseTimeDto dto) {
        return succeed(courseGroupService.lockCourseToCache(dto));
    }

    @ApiOperation("创建直播课程组-解除锁定课程时间-删除写到缓存当作锁定的课时")
    @GetMapping("/unlockCourseToCache")
    public HttpResponseResult<Object> unlockCourseToCache(@RequestParam("teacherId") Long teacherId) {
        courseGroupService.unlockCourseToCache(teacherId);
        return succeed();
    }



    @ApiOperation("直播课拥有的声部信息")
    @GetMapping("/liveSubject")
    public HttpResponseResult<List<Subject>> liveSubject() {
        List<Subject> subjectList = courseGroupService.liveSubject();
        return succeed(subjectList);
    }

    @ApiOperation("学生端我的课程组")
    @PostMapping("/myCourseGroup")
    public HttpResponseResult<PageInfo<CourseGroupWrapper.StudentCourseGroupDto>> myCourseGroup(@RequestBody CourseGroupWrapper.TeacherCourseGroupQuery query) {
        query.setStudentId(sysUserService.getUserId());
        return succeed(PageUtil.pageInfo(courseGroupService.studentCourseGroup(PageUtil.getPage(query), query)));
    }

    @ApiOperation("学生端获取单个课程组")
    @PostMapping("/getCourseGroup")
    public HttpResponseResult<CourseGroupWrapper.StudentCourseGroupDto> getCourseGroup(Long courseGroupId) {
        CourseGroupWrapper.TeacherCourseGroupQuery query = new CourseGroupWrapper.TeacherCourseGroupQuery();
        query.setCourseGroupId(courseGroupId);
        IPage<CourseGroupWrapper.StudentCourseGroupDto> pages = courseGroupService.studentCourseGroup(PageUtil.getPage(query), query);
        return succeed(pages.getRecords().get(0));
    }
}

