package com.yonge.cooleshow.student.controller.open;

import com.yonge.cooleshow.biz.dal.entity.Subject;
import com.yonge.cooleshow.biz.dal.queryInfo.SubjectQueryInfo;
import com.yonge.cooleshow.biz.dal.service.SubjectService;
import com.yonge.cooleshow.common.controller.BaseController;
import com.yonge.cooleshow.common.entity.HttpResponseResult;
import com.yonge.toolset.base.page.PageInfo;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.Map;
import java.util.stream.Collectors;

@RequestMapping("${app-config.url.student:}/open/subject")
@Api(tags = "声部服务")
@RestController
public class OpenSubjectController extends BaseController {

    @Autowired
    private SubjectService subjectService;

    @ApiOperation(value = "根据声部编号查询声部")
    @GetMapping("/get/{id}")
    public Object get(@ApiParam(value = "声部编号", required = true) @PathVariable("id") Long id) {
        return succeed(subjectService.get(id));
    }

    @ApiOperation(value = "分页查询声部树状列表")
    @PostMapping("/queryPageTree")
    public HttpResponseResult<PageInfo<Subject>> queryPageTree(@RequestBody SubjectQueryInfo queryInfo) {
        return succeed(subjectService.queryPageTree(queryInfo));
    }

    @ApiOperation(value = "分页查询声部列表")
    @GetMapping("/queryPage")
    public HttpResponseResult<PageInfo<Subject>> queryPage(SubjectQueryInfo queryInfo) {
        PageInfo<Subject> pageInfo = subjectService.queryPage(queryInfo);

        if (pageInfo.getRows().size() == 0) {
            return succeed(pageInfo);
        }

        Map<Long, Subject> map =
                subjectService.findBySubjectByIdList(pageInfo.getRows().stream().map(Subject::getParentSubjectId).collect(Collectors.toList())).stream()
                .collect(Collectors.toMap(Subject::getId, t -> t));

        pageInfo.getRows().forEach(row -> {
            if (row.getParentSubjectId() != null && row.getParentSubjectId() > 0) {
                row.setParentSubjectName(map.get(row.getParentSubjectId()).getName());
            }
        });
        return succeed(pageInfo);
    }
}
