package com.yonge.cooleshow.teacher.controller;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.yonge.cooleshow.biz.dal.dto.CourseHomeworkReviewDto;
import com.yonge.cooleshow.biz.dal.dto.CourseHomeworkSaveDto;
import com.yonge.cooleshow.biz.dal.dto.CourseScheduleHomeworkSearch;
import com.yonge.cooleshow.biz.dal.dto.search.HomeworkSearch;
import com.yonge.cooleshow.biz.dal.entity.TeacherAttendance;
import com.yonge.cooleshow.biz.dal.enums.ClientEnum;
import com.yonge.cooleshow.biz.dal.enums.CourseScheduleEnum;
import com.yonge.cooleshow.biz.dal.service.*;
import com.yonge.cooleshow.biz.dal.vo.CountVo;
import com.yonge.cooleshow.biz.dal.vo.CourseHomeworkDetailVo;
import com.yonge.cooleshow.biz.dal.vo.CourseHomeworkVo;
import com.yonge.cooleshow.biz.dal.vo.CourseScheduleHomeworkVo;
import com.yonge.cooleshow.common.controller.BaseController;
import com.yonge.cooleshow.common.entity.HttpResponseResult;
import com.yonge.cooleshow.common.enums.YesOrNoEnum;
import com.yonge.toolset.base.page.PageInfo;
import com.yonge.toolset.mybatis.support.PageUtil;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.apache.commons.collections.CollectionUtils;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import javax.validation.Valid;
import java.util.ArrayList;
import java.util.List;
import java.util.Objects;

/**
 * Description 老师课后作业相关接口
 *
 * @author liujunchi
 * @date 2022-04-13
 */
@Api(tags = "老师课后作业相关接口")
@RestController
@RequestMapping("${app-config.url.teacher:}/homework")
public class CourseHomeworkController extends BaseController {

    @Resource
    private SysUserService sysUserService;
    @Resource
    private CourseScheduleService courseScheduleService;
    @Resource
    private CourseHomeworkService courseHomeworkService;
    @Resource
    private ImGroupService imGroupService;
    @Resource
    private TeacherAttendanceService teacherAttendanceService;

    @ApiOperation(value = "未布置的课后作业数量")
    @GetMapping(value="/count")
    public HttpResponseResult<CountVo> countTeacherNoDecorateHomework() {
        HomeworkSearch homeworkSearch = new HomeworkSearch();
        // 默认查当前老师，陪练课 课程状态为完成， 没有学生课程记录
        homeworkSearch.setTeacherId(sysUserService.getUserId());
        homeworkSearch.setCourseStatus(CourseScheduleEnum.COMPLETE);
        List<CourseScheduleEnum> list = new ArrayList<>();
        list.add(CourseScheduleEnum.PIANO_ROOM_CLASS);
        list.add(CourseScheduleEnum.PRACTICE);
        homeworkSearch.setCourseType(list);
        homeworkSearch.setDecorate(YesOrNoEnum.NO);
        Integer integer = courseScheduleService.countTeacherNoDecorateHomework(homeworkSearch);
        CountVo countVo = new CountVo();
        countVo.setCount(integer);
        return succeed(countVo);
    }

    @ApiOperation(value = "首页-我的课程-课程详情(趣纠课)-课后作业信息详情",notes = "传入课程编号ID")
    @GetMapping(value = "/detail/{courseId}/{studentId}")
    public HttpResponseResult<CourseHomeworkDetailVo> detail(@ApiParam(value = "课程编号ID", required = true)
                                                                 @PathVariable("courseId") Long courseId,
                                                             @PathVariable("studentId") Long studentId) {
        Long userId = sysUserService.getUserId();
        HttpResponseResult info = checkCourseSchedule(courseId,userId);
        if (info != null) return info;
        CourseHomeworkDetailVo detailVo = courseHomeworkService.getCourseHomeworkDetailByCourseId(courseId, studentId);
        if (detailVo != null) {
            detailVo.setImUserId(imGroupService.getImUserId(String.valueOf(detailVo.getStudentId()),ClientEnum.STUDENT.name()));
            //获取老师考勤信息
            TeacherAttendance one = teacherAttendanceService.lambdaQuery().eq(TeacherAttendance::getCourseScheduleId, courseId)
                    .eq(TeacherAttendance::getTeacherId, userId).last("limit 1").one();
            detailVo.setAttendanceStatus(Objects.nonNull(one));
        }

        return succeed(detailVo);
    }


    @ApiOperation(value = "首页-我的课程-课程详情(琴房)-课后作业列表",notes = "传入课程编号ID")
    @PostMapping(value = "/detail/list")
    public HttpResponseResult<CourseScheduleHomeworkVo> detail(@RequestBody @Valid CourseScheduleHomeworkSearch query) {
        Long userId = sysUserService.getUserId();
        HttpResponseResult info = checkCourseSchedule(query.getCourseId(),userId);
        if (info != null) return info;
        CourseScheduleHomeworkVo scheduleHomeworkVo = courseHomeworkService.getCourseHomeworkDetailByCourseId(query);
        if (scheduleHomeworkVo != null) {
            //获取老师考勤信息
            TeacherAttendance one = teacherAttendanceService.lambdaQuery().eq(TeacherAttendance::getCourseScheduleId, query.getCourseId())
                    .eq(TeacherAttendance::getTeacherId, userId).last("limit 1").one();
            scheduleHomeworkVo.setAttendanceStatus(Objects.nonNull(one));
        }
        return succeed(scheduleHomeworkVo);
    }

    private HttpResponseResult checkCourseSchedule(Long courseId,Long userId) {
        if (!courseScheduleService.checkTeacherCourseSchedule(userId, courseId)) {
            return failed("老师只能看自己的课程详情");
        }
        return null;
    }

    @ApiOperation(value = "趣纠课-布置作业", httpMethod = "POST", consumes = "application/json", produces = "application/json")
    @PostMapping(value = "/decorate", consumes = "application/json", produces = "application/json")
    public HttpResponseResult<Boolean> decorateCourseHomework(@Valid @RequestBody CourseHomeworkSaveDto saveDto) {
        Long userId = sysUserService.getUserId();
        HttpResponseResult info = checkCourseSchedule(saveDto.getCourseScheduleId(),userId);
        if (info != null) return info;
        saveDto.setTeacherId(userId);
        return succeed(courseHomeworkService.decorateCourseHomework(saveDto));
    }

    @ApiOperation(value = "作业点评", httpMethod = "POST", consumes = "application/json", produces = "application/json")
    @PostMapping(value = "/review", consumes = "application/json", produces = "application/json")
    public HttpResponseResult<Boolean> reviewCourseHomework(@Valid @RequestBody CourseHomeworkReviewDto reviewDto) {
        Long userId = sysUserService.getUserId();
        HttpResponseResult info = checkCourseSchedule(reviewDto.getCourseScheduleId(),userId);
        if (info != null) return info;
        reviewDto.setTeacherId(userId);
        return succeed(courseHomeworkService.reviewCourseHomework(reviewDto));
    }

    @ApiOperation(value = "课后作业-列表", httpMethod = "POST", consumes = "application/json", produces = "application/json")
    @PostMapping(value = "/list", consumes = "application/json", produces = "application/json")
    public HttpResponseResult<PageInfo<CourseHomeworkVo>> list(@Valid @RequestBody HomeworkSearch query) {
        query.setTeacherId(sysUserService.getUserId());
        query.setCourseStatus(CourseScheduleEnum.COMPLETE);
        List<CourseScheduleEnum> list = new ArrayList<>();
        list.add(CourseScheduleEnum.PIANO_ROOM_CLASS);
        list.add(CourseScheduleEnum.PRACTICE);
        list.add(CourseScheduleEnum.VIP);
        list.add(CourseScheduleEnum.GROUP);
        query.setCourseType(list);
        query.setClientType("TEACHER");

        IPage<CourseHomeworkVo> page = courseHomeworkService.selectPage(PageUtil.getPage(query), query);
        if (CollectionUtils.isNotEmpty(page.getRecords())) {
            for (CourseHomeworkVo item : page.getRecords()) {
                item.setImUserId(imGroupService.getImUserId(String.valueOf(item.getStudentId()),ClientEnum.STUDENT.name()));
            }
        }

        return succeed(PageUtil.pageInfo(page));
    }


}
