package com.yonge.cooleshow.teacher.controller;


import com.baomidou.mybatisplus.core.metadata.IPage;
import com.yonge.cooleshow.biz.dal.dto.CheckCourseTimeDto;
import com.yonge.cooleshow.biz.dal.dto.LiveCourseGroupDto;
import com.yonge.cooleshow.biz.dal.entity.CourseGroup;
import com.yonge.cooleshow.biz.dal.entity.CourseTimeEntity;
import com.yonge.cooleshow.biz.dal.enums.CourseScheduleEnum;
import com.yonge.cooleshow.biz.dal.service.AppVersionInfoService;
import com.yonge.cooleshow.biz.dal.service.CourseGroupService;
import com.yonge.cooleshow.biz.dal.service.SysUserService;
import com.yonge.cooleshow.biz.dal.support.WrapperUtil;
import com.yonge.cooleshow.biz.dal.vo.CourseGroupVo;
import com.yonge.cooleshow.biz.dal.vo.CourseInfoVo;
import com.yonge.cooleshow.biz.dal.vo.LiveCourseInfoVo;
import com.yonge.cooleshow.biz.dal.wrapper.course.CourseGroupWrapper;
import com.yonge.cooleshow.common.controller.BaseController;
import com.yonge.cooleshow.common.entity.HttpResponseResult;
import com.yonge.cooleshow.common.enums.YesOrNoEnum;
import com.yonge.toolset.base.page.PageInfo;
import com.yonge.toolset.mybatis.support.PageUtil;
import io.swagger.annotations.*;
import org.apache.commons.collections.MapUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import java.math.BigDecimal;
import java.util.List;
import java.util.Map;
import java.util.Optional;

/**
 * 课程组表(CourseGroup)表控制层
 *
 * @author hgw
 * @since 2022-03-18 15:29:10
 */
@Api(tags = "课程组表")
@RestController
@RequestMapping("${app-config.url.teacher:}/courseGroup")
public class TeacherCourseGroupController extends BaseController {
    @Resource
    private CourseGroupService courseGroupService;

    @Resource
    private SysUserService sysUserService;

    @Resource
    private AppVersionInfoService appVersionInfoService;
    @ApiOperation("直播课详情")
    @GetMapping("/queryLiveCourseInfo")
    public HttpResponseResult<LiveCourseInfoVo> queryLiveCourseInfo(@ApiParam(value = "课程组id", required = true) @RequestParam(value = "groupId") Long groupId) {
        return succeed(courseGroupService.queryLiveCourseInfo(groupId));
    }

    @ApiOperation("课程组详情")
    @GetMapping("/queryCourseInfo")
    public HttpResponseResult<CourseInfoVo> queryCourseInfo(@ApiParam(value = "课程组id", required = true) @RequestParam(value = "groupId") Long groupId) {
        return succeed(courseGroupService.queryCourseInfo(groupId));
    }

    @ApiImplicitParams({
            @ApiImplicitParam(name = "teacherId", dataType = "Long", value = "老师id"),
            @ApiImplicitParam(name = "search", dataType = "String", value = "关键字"),
            @ApiImplicitParam(name = "groupStatus", dataType = "String", value = "课程组状态  ING(进行中)  NOT_SALE(未开售,未上架) APPLY(报名中,销售中) COMPLETE(已完成) OUT_SALE(已下架)"),
            @ApiImplicitParam(name = "page", dataType = "Integer", value = "页数"),
            @ApiImplicitParam(name = "rows", dataType = "Integer", value = "每页数量"),
            @ApiImplicitParam(name = "courseType", dataType = "String", value = "课程类型"),
            @ApiImplicitParam(name = "myself", dataType = "Boolean", value = "查看我自己的直播课"),
            @ApiImplicitParam(name = "version", dataType = "String", value = "版本"),
            @ApiImplicitParam(name = "platform", dataType = "Integer", value = "平台"),
    })
    @ApiOperation("分页查询直播课、小组课 课程组列表")
    @PostMapping("/queryPageCourseGroup")
    public HttpResponseResult<PageInfo<CourseGroupVo>> queryPageLiveCourseGroup(@RequestBody Map<String, Object> param) {


        Boolean myself = MapUtils.getBoolean(param,"myself");
        if (myself == null || myself) {
            Long teacherId = WrapperUtil.toLong(param, "teacherId");
            teacherId = Optional.ofNullable(teacherId).orElseGet(() -> sysUserService.getUserId());
            param.put("teacherId", teacherId);
        } else {
            YesOrNoEnum auditVersion = YesOrNoEnum.NO;
            if (param.get("platform") != null && param.get("version") != null) {
                auditVersion = appVersionInfoService.getAppAuditVersion(param.get("platform").toString(),param.get("version").toString());
            }
            param.put("auditVersion",auditVersion);
            param.put("groupStatus","APPLY");

        }


        return succeed(courseGroupService.queryPageLiveCourseGroup(param));
    }



    @ApiOperation("创建直播课程组-新增课程组")
    @PostMapping("/addLiveCourse")
    public HttpResponseResult<Object> addLiveCourse(@RequestBody LiveCourseGroupDto dto) {

        YesOrNoEnum appAuditVersion = appVersionInfoService.getAppAuditVersion(dto.getPlatform(), dto.getVersion());
        if (appAuditVersion == YesOrNoEnum.NO && dto.getCoursePrice().compareTo(BigDecimal.ZERO) <= 0) {
            return failed("课程价格不能为0元");
        }
        if(StringUtils.isEmpty(dto.getCourseType())){
            dto.setCourseType(CourseScheduleEnum.LIVE.getCode());
        }
        courseGroupService.addLiveCourse(dto);
        return succeed();
    }


    @ApiOperation("修改直播课程组")
    @PostMapping("/updateLiveCourse")
    public HttpResponseResult<Object> updateLiveCourse(@RequestBody LiveCourseGroupDto dto) {
        courseGroupService.updateLiveCourse(dto);
        return succeed();
    }


    @ApiOperation("创建直播课程组-锁定课程时间-将课时写到缓存当作锁定的时间")
    @PostMapping("/lockCourseToCache")
    public HttpResponseResult<List<CourseTimeEntity>> lockCourseToCache(@RequestBody CheckCourseTimeDto dto) {
        return succeed(courseGroupService.lockCourseToCache(dto));
    }

    @ApiOperation("创建直播课程组-解除锁定课程时间-删除写到缓存当作锁定的课时")
    @GetMapping("/unlockCourseToCache")
    public HttpResponseResult<Object> unlockCourseToCache(@RequestParam("teacherId") Long teacherId) {
        courseGroupService.unlockCourseToCache(teacherId);
        return succeed();
    }

    @ApiOperation("取消课程组-下架课程组")
    @GetMapping("/cancelCourseGroup")
    public HttpResponseResult<Object> cancelCourseGroup(@RequestParam("groupId") Long groupId) {
        courseGroupService.cancelCourseGroup(groupId);
        return succeed();
    }

    @ApiOperation("获取老师排直播课时锁定的课时")
    @GetMapping("/getLockCache")
    public HttpResponseResult<Object> getLockCache(@RequestParam("teacherId") Long teacherId) {
        return succeed(courseGroupService.getLiveLockTimeCache(teacherId).get(teacherId));
    }

    @ApiOperation("老师端我的课程组")
    @PostMapping("/myCourseGroup")
    public HttpResponseResult<PageInfo<CourseGroupWrapper.TeacherCourseGroupDto>> myCourseGroup(@RequestBody CourseGroupWrapper.TeacherCourseGroupQuery query) {
        query.setTeacherId(sysUserService.getUserId());
        IPage<CourseGroupWrapper.TeacherCourseGroupDto> pages = courseGroupService.teacherCourseGroup(PageUtil.getPage(query), query);
        return succeed(PageUtil.pageInfo(pages));
    }

    @ApiOperation("老师端获取单个课程组")
    @PostMapping("/getCourseGroup")
    public HttpResponseResult<CourseGroupWrapper.TeacherCourseGroupDto> getCourseGroup(Long courseGroupId) {
        CourseGroup courseGroup = courseGroupService.getById(courseGroupId);
        if (courseGroup == null) {
            return failed("课程组不存在");
        }
        CourseGroupWrapper.TeacherCourseGroupQuery query = new CourseGroupWrapper.TeacherCourseGroupQuery();
        query.setCourseGroupId(courseGroupId);
        query.setCourseType(courseGroup.getType());
        IPage<CourseGroupWrapper.TeacherCourseGroupDto> pages = courseGroupService.teacherCourseGroup(PageUtil.getPage(query), query);
        return succeed(pages.getRecords().get(0));
    }

    @ApiOperation("获取课程组关联的学员")
    @PostMapping("/queryStudentByGroupId")
    public HttpResponseResult<List<CourseGroupWrapper.CourseStudentVo>> queryStudentByGroupId(Long groupId) {
        return succeed(courseGroupService.queryStudentByGroupId(groupId));
    }

    @ApiOperation("修改课程规划")
    @PostMapping("/updateCoursePlan")
    public HttpResponseResult<Object> updateCoursePlan(Long groupId, String coursePlan) {
        courseGroupService.updateCoursePlan(groupId, coursePlan);
        return succeed();
    }
}

