package com.yonge.cooleshow.teacher.controller;

import com.alibaba.fastjson.JSONObject;
import com.yonge.cooleshow.biz.dal.entity.LiveRoomVideo;
import com.yonge.cooleshow.biz.dal.entity.RecordNotify;
import com.yonge.cooleshow.biz.dal.service.LiveRoomVideoService;
import com.yonge.cooleshow.common.controller.BaseController;
import com.yonge.cooleshow.common.entity.HttpResponseResult;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.util.List;

/**
 * 直播视频记录(LiveRoomVideo)表控制层
 *
 * @author hgw
 * @since 2022-03-18 15:41:17
 */
@Api(tags = "直播视频记录")
@RestController
@RequestMapping("${app-config.url.teacher:}/liveRoomVideo")
public class TeacherLiveRoomVideoController extends BaseController {

    private final static Logger log = LoggerFactory.getLogger(TeacherLiveRoomVideoController.class);

    /**
     * 服务对象
     */
    @Autowired
    private LiveRoomVideoService liveRoomVideoService;

    @ApiOperation("查询直播回放")
    @GetMapping("/queryVideo")
    public HttpResponseResult<List<LiveRoomVideo>> queryVideo(@ApiParam(value = "房间uid", required = true) String roomUid) {
        return succeed(liveRoomVideoService.queryVideo(roomUid));
    }

    @ApiOperation("录制结果回调")
    @RequestMapping(value = "/recordSync")
    public void recordSync(@RequestBody String body) {
        log.info("recordSync body：{}", body);
        RecordNotify recordNotify = JSONObject.parseObject(body, RecordNotify.class);
        liveRoomVideoService.recordSync(recordNotify);
    }

}

