package com.yonge.cooleshow.teacher.controller;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.yonge.cooleshow.auth.api.client.SysUserFeignService;
import com.yonge.cooleshow.auth.api.entity.SysUser;
import com.yonge.cooleshow.biz.dal.dto.req.TotalReq;
import com.yonge.cooleshow.biz.dal.dto.search.UserAccountRecordSearch;
import com.yonge.cooleshow.biz.dal.service.UserAccountRecordService;
import com.yonge.cooleshow.biz.dal.service.UserAccountService;
import com.yonge.cooleshow.biz.dal.vo.UserAccountRecordVo;
import com.yonge.cooleshow.biz.dal.vo.UserAccountVo;
import com.yonge.cooleshow.biz.dal.vo.res.AccountShareTotal;
import com.yonge.cooleshow.biz.dal.vo.res.AccountTotal;
import com.yonge.cooleshow.common.controller.BaseController;
import com.yonge.cooleshow.common.entity.HttpResponseResult;
import com.yonge.toolset.base.exception.BizException;
import com.yonge.toolset.base.page.PageInfo;
import com.yonge.toolset.base.util.StringUtil;
import com.yonge.toolset.mybatis.support.PageUtil;
import com.yonge.toolset.utils.date.DateUtil;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.validation.Valid;
import java.time.LocalDate;
import java.time.LocalDateTime;
import java.time.LocalTime;
import java.time.temporal.TemporalAdjusters;
import java.util.Date;

@RestController
@RequestMapping("${app-config.url.teacher:}/userAccount")
@Api(value = "用户账户表", tags = "用户账户表")
public class UserAccountController extends BaseController {
    @Autowired
    private UserAccountService userAccountService;
    @Autowired
    private UserAccountRecordService userAccountRecordService;
    @Autowired
    private SysUserFeignService sysUserFeignService;

	/**
     * 查询单条
     */
    @GetMapping("/detail")
    @ApiOperation(value = "详情")
    public HttpResponseResult<UserAccountVo> detail() {
        SysUser user = sysUserFeignService.queryUserInfo();
        if (user == null || null == user.getId()) {
            return failed(HttpStatus.FORBIDDEN, "请登录");
        }
    	return succeed(userAccountService.detail(user.getId()));
	}

    /**
     * 查询分页
     */
    @PostMapping("/page")
    @ApiOperation(value = "查询分页", notes = "传入userAccountRecordSearch")
    public HttpResponseResult<PageInfo<UserAccountRecordVo>> page(@RequestBody UserAccountRecordSearch query) {
        SysUser user = sysUserFeignService.queryUserInfo();
        if (user == null || null == user.getId()) {
            return failed(HttpStatus.FORBIDDEN, "请登录");
        }
        query.setUserId(user.getId());

        if (StringUtil.isEmpty(query.getSearchDate())) {
            query.setSearchDate(DateUtil.format(new Date(), "yyyy-MM"));
        }
        String[] classDateSp = query.getSearchDate().split("-");
        try {
            LocalDate date = LocalDate.of(Integer.parseInt(classDateSp[0]), Integer.parseInt(classDateSp[1]), 1);

            LocalDateTime firstDay = LocalDateTime.of(date.with(TemporalAdjusters.firstDayOfMonth()), LocalTime.MIN);
            LocalDateTime lastDay = LocalDateTime.of(date.with(TemporalAdjusters.lastDayOfMonth()), LocalTime.MAX);

            query.setStartTime(firstDay);
            query.setEndTime(lastDay);
        } catch (Exception e) {
            throw new BizException("查询时间格式不正确 [" + query.getSearchDate() + "]");
        }

        IPage<UserAccountRecordVo> pages = userAccountRecordService.appSelectPage(PageUtil.getPage(query), query);
        return succeed(PageUtil.pageInfo(pages));
    }

    @ApiOperation(value = "收入数据统计")
    @PostMapping("/accountTotal")
    public HttpResponseResult<AccountTotal> accountTotal(@Valid @RequestBody TotalReq query) {
        SysUser user = sysUserFeignService.queryUserInfo();
        if (user == null || null == user.getId()) {
            return failed(HttpStatus.FORBIDDEN, "请登录");
        }
        query.setUserId(user.getId());
        return userAccountService.accountTotal(query);
    }

    @ApiOperation(value = "酷乐秀推广计划-统计")
    @PostMapping("/accountShareTotal")
    public HttpResponseResult<AccountShareTotal> accountShareTotal() {
        SysUser user = sysUserFeignService.queryUserInfo();
        if (user == null || null == user.getId()) {
            return failed(HttpStatus.FORBIDDEN, "请登录");
        }
        TotalReq query = new TotalReq();
        query.setUserId(user.getId());
        return userAccountService.accountShareTotal(query);
    }

    @PostMapping("/sharePage")
    @ApiOperation(value = "酷乐秀推广计划-分页", notes = "传入userAccountRecordSearch")
    public HttpResponseResult<PageInfo<UserAccountRecordVo>> sharePage(@RequestBody UserAccountRecordSearch query) {
        SysUser user = sysUserFeignService.queryUserInfo();
        if (user == null || null == user.getId()) {
            return failed(HttpStatus.FORBIDDEN, "请登录");
        }
        query.setUserId(user.getId());
        if (StringUtil.isEmpty(query.getSearchDate())) {
            query.setSearchDate(DateUtil.format(new Date(), "yyyy-MM"));
        }
        String[] classDateSp = query.getSearchDate().split("-");
        try {
            LocalDate date = LocalDate.of(Integer.parseInt(classDateSp[0]), Integer.parseInt(classDateSp[1]), 1);

            LocalDateTime firstDay = LocalDateTime.of(date.with(TemporalAdjusters.firstDayOfMonth()), LocalTime.MIN);
            LocalDateTime lastDay = LocalDateTime.of(date.with(TemporalAdjusters.lastDayOfMonth()), LocalTime.MAX);

            query.setStartTime(firstDay);
            query.setEndTime(lastDay);
        } catch (Exception e) {
            throw new BizException("查询时间格式不正确 [" + query.getSearchDate() + "]");
        }
        IPage<UserAccountRecordVo> pages = userAccountRecordService.sharePage(PageUtil.getPage(query), query);
        return succeed(PageUtil.pageInfo(pages));
    }
}