package com.yonge.cooleshow.tenant.controller;

import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.dayaedu.cbs.openfeign.client.CoursewareFeignService;
import com.dayaedu.cbs.openfeign.wrapper.courseware.CbsLessonCoursewareWrapper;
import com.microsvc.toolkit.common.response.paging.QueryInfo;
import com.yonge.cooleshow.auth.api.client.SysUserFeignService;
import com.yonge.cooleshow.auth.api.entity.SysUser;
import com.yonge.cooleshow.biz.dal.enums.ClientEnum;
import com.yonge.cooleshow.biz.dal.service.TenantAlbumMusicService;
import com.yonge.cooleshow.biz.dal.wrapper.LessonCoursewareDetailWrapper;
import com.yonge.cooleshow.biz.dal.wrapper.LessonCoursewareWrapper;
import com.yonge.cooleshow.biz.dal.wrapper.TenantAlbumMusicWrapper;
import com.yonge.cooleshow.common.controller.BaseController;
import com.yonge.cooleshow.common.entity.HttpResponseResult;
import com.yonge.toolset.base.page.PageInfo;
import com.yonge.toolset.mybatis.support.PageUtil;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.http.HttpStatus;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import javax.validation.Valid;
import java.util.ArrayList;
import java.util.List;

/**
 * 专辑表 web 控制层
 * @author yzp
 * @date 2022-03-26 00:21:46
 * @version v1.0
 **/
@RestController
@RequestMapping("${app-config.url.tenant:}/tenantAlbumMusic")
@Api(tags = "机构专辑曲目 API接口")
public class TenantAlbumSheetController extends BaseController {

    @Resource
    private TenantAlbumMusicService tenantAlbumMusicService;

    @Resource
    private SysUserFeignService sysUserFeignService;

    @Resource
    private CoursewareFeignService coursewareFeignService;

    @ApiOperation(value = "查询条件")
    @PostMapping("/selectCondition")
    public HttpResponseResult<TenantAlbumMusicWrapper.TenantAlbumMusicSelectData> selectCondition(@RequestBody TenantAlbumMusicWrapper.TenantAlbumMusicSelect query) {
        SysUser user = sysUserFeignService.queryUserInfo();
        if (user == null || null == user.getId()) {
            return failed(HttpStatus.FORBIDDEN, "请登录");
        }
//        query.setUserId(user.getId());
        /*if(StringUtils.isBlank(query.getSubjectId())){
        	Student student = studentService.getById(user.getId());
        	query.setSubjectId(student.getSubjectId());
        }*/
        return succeed(tenantAlbumMusicService.getTenantAlbumMusicQuery(query));
    }



    @ApiOperation(value = "分页查询")
    @PostMapping("/page")
    public HttpResponseResult<PageInfo<TenantAlbumMusicWrapper.StudentTenantAlbumMusic>> page(@RequestBody @Validated TenantAlbumMusicWrapper.StudentTenantAlbumMusicQuery query) {
        SysUser user = sysUserFeignService.queryUserInfo();
        if (user == null || null == user.getId()) {
            return failed(HttpStatus.FORBIDDEN, "请登录");
        }
//        query.setTenantId(Optional.ofNullable(user.getTenantId()).orElse(-1L));
        query.setClientType(ClientEnum.TENANT);
        IPage<TenantAlbumMusicWrapper.StudentTenantAlbumMusic> page = tenantAlbumMusicService.selectPage(QueryInfo.getPage(query), query);
        return succeed((PageUtil.pageInfo(page)));
    }


    @ApiOperation(value = "获取课件详情")
    @PostMapping("/getLessonCoursewareDetail")
    public HttpResponseResult<LessonCoursewareWrapper.LessonCourseware> getLessonCoursewareDetail(@RequestBody @Valid LessonCoursewareWrapper.LessonCourseQuery query){
        query.setClient(ClientEnum.TENANT);
        SysUser sysUser = sysUserFeignService.queryUserInfo();
        if (sysUser == null || null == sysUser.getId()) {
            return failed("用户信息获取失败");
        }
        query.setUserId(sysUser.getId());
        LessonCoursewareWrapper.LessonCourseware lessonCourseware = tenantAlbumMusicService.getLessonCoursewareDetail(query);
        return succeed(lessonCourseware);
    }


    @ApiOperation(value = "获取课件关联的课程列表")
    @GetMapping("/getLessonCoursewareCourseList/{id}")
    public HttpResponseResult<List<LessonCoursewareWrapper.CourseScheduleCoursewareDetail>> getLessonCoursewareCourseList(@PathVariable("id") Long id,String search,Boolean detailFlag){
        CbsLessonCoursewareWrapper.CoursewareDetailQuery query = new CbsLessonCoursewareWrapper.CoursewareDetailQuery();
        query.setId(id);
        query.setSearch(search);
        query.setDetailFlag(detailFlag);
        List<CbsLessonCoursewareWrapper.CourseScheduleCoursewareDetail> courseScheduleCoursewareDetails = coursewareFeignService.coursewareDetail(query).feignData();
        if (CollectionUtils.isNotEmpty(courseScheduleCoursewareDetails)) {
            //过滤酷乐秀相关的权限
            courseScheduleCoursewareDetails.forEach(tenantAlbumMusicService::filterLessonCourseDetail);
            courseScheduleCoursewareDetails.removeIf(e->!tenantAlbumMusicService.filterAndUpdateKnowledgePointList(e));
            String jsonString = JSONObject.toJSONString(courseScheduleCoursewareDetails);
            List<LessonCoursewareWrapper.CourseScheduleCoursewareDetail> courseScheduleCoursewareDetailList = JSONObject
                .parseArray(jsonString, LessonCoursewareWrapper.CourseScheduleCoursewareDetail.class);
            return succeed(courseScheduleCoursewareDetailList);
        }
        return succeed(new ArrayList<>());
    }

    @ApiOperation(value = "获取课程详情")
    @GetMapping("/getLessonCourseDetail/{id}")
    public HttpResponseResult<LessonCoursewareDetailWrapper.LessonCoursewareDetail> getLessonCourseDetail(@PathVariable("id") String id){
        return succeed(tenantAlbumMusicService.getLessonCourseDetail(id));
    }


}
