package com.yonge.cooleshow.website.controller;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.yonge.cooleshow.auth.api.client.SysUserFeignService;
import com.yonge.cooleshow.auth.api.entity.SysUser;
import com.yonge.cooleshow.biz.dal.dto.search.MusicSheetRelatedQueryInfo;
import com.yonge.cooleshow.biz.dal.dto.search.MusicSheetSearch;
import com.yonge.cooleshow.biz.dal.dto.search.StudentMusicSheetSearch;
import com.yonge.cooleshow.biz.dal.entity.MusicSheet;
import com.yonge.cooleshow.biz.dal.enums.AuthStatusEnum;
import com.yonge.cooleshow.biz.dal.enums.ClientEnum;
import com.yonge.cooleshow.biz.dal.enums.SourceTypeEnum;
import com.yonge.cooleshow.biz.dal.enums.album.PurchaseRecordTypeEnum;
import com.yonge.cooleshow.biz.dal.service.MusicSheetAuthRecordService;
import com.yonge.cooleshow.biz.dal.service.MusicSheetService;
import com.yonge.cooleshow.biz.dal.vo.MusicSheetVo;
import com.yonge.cooleshow.biz.dal.vo.TeacherAuditMusicSheetVo;
import com.yonge.cooleshow.common.controller.BaseController;
import com.yonge.cooleshow.common.entity.HttpResponseResult;
import com.yonge.cooleshow.common.enums.EStatus;
import com.yonge.cooleshow.common.enums.YesOrNoEnum;
import com.yonge.toolset.base.page.PageInfo;
import com.yonge.toolset.base.page.QueryInfo;
import com.yonge.toolset.base.util.StringUtil;
import com.yonge.toolset.mybatis.support.PageUtil;
import io.swagger.annotations.*;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import java.util.Optional;

/**
 * 曲谱表 web 控制层
 * @author yzp
 * @date 2022-03-26 00:21:46
 * @version v1.0
 **/
@RestController
@RequestMapping("${app-config.url.website:}/music/sheet")
@Api(tags = "曲谱接口")
public class MusicSheetController extends BaseController {
    @Resource
    private SysUserFeignService sysUserFeignService;

	@Resource
	private MusicSheetService musicSheetService;
	@Resource
	private MusicSheetAuthRecordService musicSheetAuthRecordService;

    /**
     * 曲目收藏
     */
    @PostMapping("/favorite/{id}")
    @ApiOperation(value = "曲目收藏/取消收藏")
    public HttpResponseResult<Boolean> favorite(@ApiParam(value = "曲目编号", required = true) @PathVariable("id") Long id,
                                                @ApiParam(value = "客户端类型") @RequestParam(value = "clientType", required = false, defaultValue = "STUDENT") String clientType) {
        SysUser sysUser = sysUserFeignService.queryUserInfo();
        if (sysUser == null  || sysUser.getId() == null) {
            return failed("用户信息获取失败");
        }

        if (ClientEnum.invalid(clientType)) {
            return failed("无效的客户端类型");
        }

        return succeed(musicSheetService.setFavorite(sysUser.getId(), id, ClientEnum.valueOf(clientType),"PLATFORM"));
    }


    /**
     * 我的单曲，收藏单曲
     */
    @ApiOperation(value = "学生-我的单曲")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "clientType", value = "客户端类型 TEACHER, STUDENT, 默认值: STUDENT"),
            @ApiImplicitParam(name = "gift", value = "课程赠品 0:购买 1:赠品, 默认值: 0"),
            @ApiImplicitParam(name = "page", value = "分页查询， 默认值: 1"),
            @ApiImplicitParam(name = "row", value = "当前页行数, 默认值: 10"),
    })
    @GetMapping("/my")
    public HttpResponseResult<PageInfo<MusicSheetVo>> myMusic(@RequestParam(value = "clientType", required = false, defaultValue = "STUDENT") String clientType,
                                                              @RequestParam(value = "gift", required = false, defaultValue = "0") Integer gift,
                                                              QueryInfo query) {
        SysUser sysUser = sysUserFeignService.queryUserInfo();
        if (sysUser == null  || sysUser.getId() == null) {
            return failed("用户信息获取失败");
        }

        if (ClientEnum.invalid(clientType)) {
            return failed("无效的客户端类型");
        }

        StudentMusicSheetSearch search = new StudentMusicSheetSearch();
        search.setStudentId(sysUser.getId());
        search.setAuditStatus(null);
        search.setUserType(ClientEnum.valueOf(clientType));
        search.setPurchaseType(Optional.ofNullable(search.getPurchaseType()).orElse(PurchaseRecordTypeEnum.MUSIC));
        search.setCourseGift(Optional.ofNullable(search.getCourseGift()).orElse(EStatus.DISABLE.getValue()));

        IPage<MusicSheetVo> sheetVoIPage = musicSheetService.myMusic(PageUtil.getPage(query),search);
        return succeed(PageUtil.pageInfo(sheetVoIPage));
    }



    /**
     * 我的单曲，收藏单曲
     */
    @GetMapping("/favorite")
    @ApiOperation(value = "学生-收藏单曲")
    public HttpResponseResult<PageInfo<MusicSheetVo>> favoriteMusic(@ApiParam(value = "客户端类型") @RequestParam(value = "clientType", required = false, defaultValue = "STUDENT") String clientType,
                                                                    QueryInfo query) {
        SysUser sysUser = sysUserFeignService.queryUserInfo();
        if (sysUser == null  || sysUser.getId() == null) {
            return failed("用户信息获取失败");
        }

        if (ClientEnum.invalid(clientType)) {
            return failed("无效的客户端类型");
        }

        StudentMusicSheetSearch search = new StudentMusicSheetSearch();
        search.setStudentId(sysUser.getId());
        search.setState(YesOrNoEnum.YES);
        search.setAuditStatus(null);
        IPage<MusicSheetVo> sheetVoIPage = musicSheetService.favoriteMusic(PageUtil.getPage(query),search, ClientEnum.valueOf(clientType));
        return succeed(PageUtil.pageInfo(sheetVoIPage));
    }

    @ApiOperation(value = "老师-个人中心-曲目分页查询", httpMethod="POST", consumes="application/json", produces="application/json")
    @PostMapping(value="/teacher/list", consumes="application/json", produces="application/json")
    public HttpResponseResult<TeacherAuditMusicSheetVo> list(@RequestBody MusicSheetSearch query) {
        SysUser sysUser = sysUserFeignService.queryUserInfo();
        if (sysUser == null  || sysUser.getId() == null) {
            return failed("用户信息获取失败");
        }
        // 设置只查老师用户 状态为启用
        query.setCreateBy(sysUser.getId());
        AuthStatusEnum auditStatus = query.getAuditStatus();
        if (query.getAuditStatus() != null && AuthStatusEnum.PASS.getCode().equals(query.getAuditStatus().getCode())) {
            query.setState(YesOrNoEnum.YES);
        } else if (query.getAuditStatus() != null && AuthStatusEnum.OUT_SALE.getCode().equals(query.getAuditStatus().getCode())) {
            query.setState(YesOrNoEnum.NO);
            query.setAuditStatus(AuthStatusEnum.PASS);
        }else if(query.getAuditStatus() != null && (AuthStatusEnum.UNPASS.getCode().equals(query.getAuditStatus().getCode()) ||
                AuthStatusEnum.DOING.getCode().equals(query.getAuditStatus().getCode()))){
            query.setState(YesOrNoEnum.NO);
            query.setDelFlag(true);
        }
        query.setSourceType(SourceTypeEnum.TEACHER);
        query.setMyself(true);

        TeacherAuditMusicSheetVo teacherAuditMusicSheetVo = musicSheetService.selectTeacherCount(sysUser.getId());
        if (teacherAuditMusicSheetVo == null) {
            teacherAuditMusicSheetVo = new TeacherAuditMusicSheetVo();
        }
        IPage<MusicSheetVo> musicSheetVoIPage = musicSheetService.selectPage(PageUtil.getPage(query), query);
        teacherAuditMusicSheetVo.setList(PageUtil.pageInfo(musicSheetVoIPage));
        if(auditStatus != null && !auditStatus.equals(AuthStatusEnum.PASS)){
            teacherAuditMusicSheetVo.setList(PageUtil.pageInfo(musicSheetAuthRecordService.convertMusicDetail(musicSheetVoIPage, query.getAuditStatus())));
        }
        return succeed(teacherAuditMusicSheetVo);
    }
    /**
     * 启用/停用
     */
    @PostMapping("/state/{id}")
    @ApiOperation(value = "启用/停用", notes = "传入id")
    public HttpResponseResult<Boolean> state(@ApiParam(value = "曲目编号", required = true)  @PathVariable Long id) {
        if (StringUtil.isEmpty(id)) {
            return failed("参数不能为空");
        }
        return status(musicSheetService.state(id,"", ClientEnum.TEACHER));
    }

    @ApiOperation(value = "关联的曲目列表")
    @GetMapping(value="/queryRelatedList")
    public HttpResponseResult<PageInfo<MusicSheetVo>> queryRelatedList(MusicSheetRelatedQueryInfo queryInfo) {
    	
    	IPage<MusicSheetVo> musicSheetVoIPage = musicSheetService.queryRelatedList(PageUtil.getPage(queryInfo), queryInfo);
        return succeed(PageUtil.pageInfo(musicSheetVoIPage));
    }
}
