package com.yonge.cooleshow.auth.api.client;

import com.dayaedu.cbs.openfeign.wrapper.qrcode.CbsQrCodeScanWrapper;
import com.yonge.cooleshow.auth.api.client.fallback.SysUserFeignServiceFallback;
import com.yonge.cooleshow.auth.api.dto.RealnameAuthReq;
import com.yonge.cooleshow.auth.api.dto.SysUserQueryInfo;
import com.yonge.cooleshow.auth.api.entity.SysUser;
import com.yonge.cooleshow.common.entity.HttpResponseResult;
import com.yonge.toolset.feign.config.FeignConfiguration;
import com.yonge.toolset.utils.idcard.IdcardInfoExtractor;
import io.swagger.annotations.ApiOperation;
import org.springframework.cloud.openfeign.FeignClient;
import org.springframework.http.MediaType;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestParam;

import java.util.List;
import java.util.Map;

@FeignClient(contextId = "sysUserFeignService", name = "${app-config.open-feign.auth-server.name:auth-server}", url = "${app-config.open-feign.auth-server.url:}",
		configuration = { FeignConfiguration.class }, fallback = SysUserFeignServiceFallback.class)
public interface SysUserFeignService {

	@GetMapping(value = "user/queryUserById/{userId}")
	public SysUser queryUserById(@PathVariable("userId") Long userId);

	@GetMapping(value = "user/queryUserByPhone")
	public SysUser queryUserByMobile(@RequestParam("mobile") String mobile);

	@PostMapping(value = "user/add", consumes = MediaType.APPLICATION_JSON_VALUE)
	public HttpResponseResult<Long> add(SysUser user);

	@PostMapping(value = "user/update", consumes = MediaType.APPLICATION_JSON_VALUE)
	public HttpResponseResult<Integer> update(SysUser user);

	@GetMapping(value = "queryUserInfo")
	public SysUser queryUserInfo();

	@GetMapping(value = "role/queryRoleCodeListByUserId")
	public List<String> queryRoleCodeListByUserId(@RequestParam("userId") Long userId);

	@PostMapping(value = "user/updateSysUser", consumes = MediaType.APPLICATION_JSON_VALUE)
	public HttpResponseResult<Boolean> updateSysUser(SysUser user);

	@PostMapping(value = "exit/{clientId}/{phone}", consumes = MediaType.APPLICATION_FORM_URLENCODED_VALUE)
	public HttpResponseResult<String> exitByPhone(@PathVariable("clientId") String clientId, @PathVariable("phone") String phone);

	@PostMapping(value = "user/realNameAuth")
	public HttpResponseResult<IdcardInfoExtractor> realNameAuth(@RequestBody RealnameAuthReq realNameAuthDto);

	@PostMapping(value = "user/bindOpenId")
	HttpResponseResult<Boolean> bindOpenId(@RequestBody SysUser user);

	@PostMapping(value = "/refreshToken")
	@ApiOperation(value = "刷新token")
	HttpResponseResult refreshToken(@RequestParam("refreshToken")String refreshToken,
										   @RequestParam("clientId")String clientId,
										   @RequestParam("clientSecret")String clientSecret);

	@GetMapping(value = "user/getTenantByClient")
	@ApiOperation(value = "获取机构编号")
	HttpResponseResult<Long> getTenantByClient(@RequestParam("userId")Long userId,@RequestParam("clientId")String clientId);

	@GetMapping(value = "exit")
	@ApiOperation(value = "退出登录")
	HttpResponseResult<String> logout();

	@GetMapping(value = "exit/{clientId}/{phone}", consumes = MediaType.APPLICATION_FORM_URLENCODED_VALUE)
	@ApiOperation(value = "退出登录")
	HttpResponseResult<String> logout(@PathVariable("clientId") String clientId, @PathVariable("phone") String phone);

	@PostMapping(value = "exit/{clientId}/{phone}/{openId}", consumes = MediaType.APPLICATION_FORM_URLENCODED_VALUE)
	@ApiOperation(value = "指定机构用户退出登录")
	HttpResponseResult<String> exitByPhoneAndOpenId(@PathVariable("clientId") String clientId,
														   @PathVariable("phone") String phone,
														   @PathVariable("openId") String openId);

	@PostMapping(value = "user/list")
	HttpResponseResult<List<SysUser>> page(@RequestBody SysUserQueryInfo queryInfo);

	@GetMapping("/client")
	@ApiOperation(value = "获取登录用户client")
	HttpResponseResult<String> client();

	@PostMapping(value = "open/qrcode/userInfo")
	CbsQrCodeScanWrapper.UserInfo userInfo(@RequestBody CbsQrCodeScanWrapper.QrCodeScanUserInfoReq req);

	@PostMapping(value = "open/qrcode/login")
	CbsQrCodeScanWrapper.QrCodeScanToken login(@RequestBody CbsQrCodeScanWrapper.QrCodeScanReq req);

	@PostMapping(value = "smsLogin")
	HttpResponseResult<Map<String,Object>> smsLogin(@RequestParam("phone")String phone,
													@RequestParam("smsCode")String smsCode,
													@RequestParam("loginType")String loginType,
													@RequestParam("clientId")String clientId,
													@RequestParam("clientSecret")String clientSecret);

    @ApiOperation(value = "用户注销账号校验")
    @GetMapping(value = "/user/logoffUser/{id}")
    HttpResponseResult<Void> logoffUser(@PathVariable("id") Long id);

}
