package com.yonge.cooleshow.auth.config;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Configuration;
import org.springframework.security.config.annotation.web.builders.HttpSecurity;
import org.springframework.security.oauth2.config.annotation.web.configuration.EnableResourceServer;
import org.springframework.security.oauth2.config.annotation.web.configuration.ResourceServerConfigurerAdapter;
import org.springframework.security.oauth2.config.annotation.web.configurers.ResourceServerSecurityConfigurer;

import com.yonge.cooleshow.common.security.BaseAccessDeniedHandler;
import com.yonge.cooleshow.common.security.BaseAuthenticationEntryPoint;

/**
 * 资源服务器配置
 */
@Configuration
@EnableResourceServer
public class ResourceServerConfig extends ResourceServerConfigurerAdapter {

	@Autowired
	private BaseAccessDeniedHandler baseAccessDeniedHandler;

	@Autowired
	private BaseAuthenticationEntryPoint baseAuthenticationEntryPoint;

	@Override
	public void configure(HttpSecurity http) throws Exception {
		http.csrf().disable().exceptionHandling().accessDeniedHandler(baseAccessDeniedHandler).authenticationEntryPoint(baseAuthenticationEntryPoint).and()
				.authorizeRequests()
			.antMatchers("/open/**", "/*/open/**").permitAll()
			.antMatchers(
						"/task/**",
				"/user/updatePassword",
				"/user/noAuth/queryUserByPhone",
				"/user/queryUserByPhone",
				"/user/add",
				"/user/queryUserById/*","/user/getTenantByClient").hasIpAddress("0.0.0.0/0")
				.anyRequest().authenticated().and().httpBasic();
	}

	@Override
	public void configure(ResourceServerSecurityConfigurer resources) throws Exception {
		resources.authenticationEntryPoint(baseAuthenticationEntryPoint).accessDeniedHandler(baseAccessDeniedHandler);
	}

}
