package com.yonge.cooleshow.portal.service;

import com.yonge.cooleshow.auth.api.entity.SysUser;
import com.yonge.cooleshow.mbg.model.PmsProduct;
import com.yonge.cooleshow.portal.domain.PmsPortalProductDetail;
import com.yonge.cooleshow.portal.domain.PmsProductCategoryNode;
import com.yonge.cooleshow.portal.dto.ProductSearchConditionVo;
import com.yonge.cooleshow.portal.dto.ShareProductVo;

import java.util.List;
import java.util.Map;

/**
 * 前台商品管理Service
 * Created by macro on 2020/4/6.
 */
public interface PmsPortalProductService {
    /**
     * 综合搜索商品
     */
    List<PmsProduct> search(String keyword, Long brandId, Long productCategoryId, Long productAttributeCategoryId, Integer pageNum, Integer pageSize, Integer sort);

    /**
     * 以树形结构获取所有商品分类
     */
    List<PmsProductCategoryNode> categoryTreeList();

    /**
     * 获取前台商品详情
     */
    PmsPortalProductDetail detail(Long id);

    /**
     * 获取商品列表下拉条件
     *
     * @return ProductSearchConditionVo
     */
    ProductSearchConditionVo searchCondition();

    /**
     * 老师分享商品分润
     *
     * @param sysUser 推荐人信息
     * @param productId 商品id
     */
    ShareProductVo shareProductProfit(SysUser sysUser, Long productId);

    /**
     * 商品编号名称
     *
     * @param goodsIds
     * @return
     */
    Map<Long,String> queryNameMap(List<Long> goodsIds);

    /**
     * 查询商品和商品sku库存
     *
     * @param goodsSkuId skuId
     * @return
     */
    Integer getProductStockAndSkuStock(Long goodsSkuId);

    /**
     * 锁定 解锁库存
     *
     * @param goodsSkuId
     * @param lockState
     * @return
     */
    Boolean lockStock(Long goodsSkuId, Boolean lockState);

}
