package com.yonge.cooleshow.biz.dal.dao;

import com.baomidou.mybatisplus.core.mapper.BaseMapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.yonge.cooleshow.auth.api.entity.SysUser;
import com.yonge.cooleshow.biz.dal.dto.PracticeScheduleDto;
import com.yonge.cooleshow.biz.dal.dto.UserAccountRecordDto;
import com.yonge.cooleshow.biz.dal.dto.search.HomeworkSearch;
import com.yonge.cooleshow.biz.dal.dto.search.MyCourseSearch;
import com.yonge.cooleshow.biz.dal.dto.search.PracticeTeacherSearch;
import com.yonge.cooleshow.biz.dal.entity.CourseSchedule;
import com.yonge.cooleshow.biz.dal.enums.CourseScheduleEnum;
import com.yonge.cooleshow.biz.dal.enums.course.CourseTypeEnum;
import com.yonge.cooleshow.biz.dal.queryInfo.TeacherBindingUserQueryInfo;
import com.yonge.cooleshow.biz.dal.vo.*;
import com.yonge.cooleshow.biz.dal.vo.userBindingTeacher.UserBindingCourseWrapper;
import com.yonge.cooleshow.biz.dal.vo.userBindingTeacher.UserBindingTeacherWrapper;
import com.yonge.cooleshow.common.enums.YesOrNoEnum;
import org.apache.ibatis.annotations.Param;

import java.math.BigDecimal;
import java.util.Date;
import java.util.List;
import java.util.Map;
import java.util.Set;

/**
 * 老师课程表(CourseSchedule)表数据库访问层
 *
 * @author hgw
 * @since 2022-03-18 15:29:11
 */
public interface CourseScheduleDao extends BaseMapper<CourseSchedule> {

    int insertBatch(@Param("entities") List<CourseSchedule> entities);

    /***
     * 查询所有老师课时统计
     * @author liweifan
     * @param: userId
     * @updateTime 2022/5/13 17:51
     * @return: java.util.List<com.yonge.cooleshow.biz.dal.vo.TeacherTotalVo>
     */
    List<TeacherTotalVo> queryTeacherTotal(@Param("userId") Long userId);

    /***
     * 查询所有学生课时统计
     * @author liweifan
     * @param: userId
     * @updateTime 2022/5/13 17:51
     * @return: java.util.List<com.yonge.cooleshow.biz.dal.vo.StudentTotalVo>
     */
    List<StudentTotalVo> queryStudentTotal(@Param("userId") Long userId);

    /**
     * 根据学生id查询课程
     *
     * @param param 传入参数
     *              <p> - studentId 学生id
     *              <p> - status 课程状态 NOT_START未开始 ING进行中 COMPLETE已完成
     *              <p> - statusList 课程状态集合 NOT_START未开始 ING进行中 COMPLETE已完成
     *              <p> - classDate  年月日
     *              <p> - startClassDate 开始时间-年月日
     *              <p> - endClassDate 结束时间-年月日
     *              <p> - greaterDate 上课时间大于该时间-年月日时分秒
     */
    List<CourseSchedule> queryStudentCourse(@Param("param") Map<String, Object> param);

    /**
     * 查询老师直播课程
     *
     * @param param 传入参数
     *              <p> - teacherId 老师id
     *              <p> - status 课程状态 NOT_START未开始 ING进行中 COMPLETE已完成
     *              <p> - type  PRACTICE 趣纠课 LIVE 直播课
     *              <p> - startDate 开始时间
     *              <p> - endDate 结束时间
     *              <p> - subjectId 声部id
     */
    <T> IPage<T> queryLiveTeacherCourse(Page<T> page, @Param("param") Map<String, Object> param);

    //根据老师id查询购课学员
    List<MyCourseVo> queryTeacherPracticeCourse(IPage page, @Param("param") MyCourseSearch query);


    /**
     * 老师未布置的课程作业数量
     * <p>
     * 趣纠课 课程状态为完成，有学生考勤记录 没有布置作业
     *
     * @param param 查询条件
     * @return number
     */
    Integer countTeacherNoDecorateHomework(@Param("param") HomeworkSearch param);

    //查询老师约课日历
    Set<String> queryCourseSchedule(MyCourseSearch search);

    //根据学生id查询老师
    List<MyCourseVo> queryStudentPracticeCourse(IPage<MyCourseVo> page, @Param("param") MyCourseSearch search);

    //查询学生约课日历
    Set<String> queryCourseScheduleStudent(MyCourseSearch search);

    //老师端-课表-日历-用户
    List<CourseStudent> queryCourseUser(@Param("param") MyCourseSearch search);

    //学生端-首页-趣纠课老师列表
    List<PracticeTeacherVo> teacherList(IPage<PracticeTeacherVo> page, @Param("param") PracticeTeacherSearch search);

    List<Long> teacherIdList();

    //学生端-课表-日历-用户
    List<CourseStudent> queryCourseTeacher(@Param("param") MyCourseSearch search);

    //新增组
    void addCourseGroup(PracticeScheduleDto scheduleDto);

    /**
     * 学生-查询直播课
     *
     * @param param <p> - studentId 学生id
     *              <p> - courseState 课程状态 NOT_START未开始 ING进行中 COMPLETE已完成 CANCEL已取消
     *              <p> - startDate 开始时间
     *              <p> - endDate 结束时间
     *              <p> - subjectId 声部id
     *              <p> - type PRACTICE 趣纠课 LIVE 直播课
     *              <p> - orderState OrderStatusEnum支付状态
     */
    <T> IPage<T> queryStudentLiveCourse(Page<T> page, @Param("param") Map<String, Object> param);

    //更新lock
    void updateLock(List<Long> list);

    //根据课程id修改时间
    void courseAdjust(CourseAdjustVo adjustVo);

    //学生端-首页-直播课
    List<StudentHomePage.Live> selectLive(@Param("appAuditVersion") YesOrNoEnum appAuditVersion, @Param("subjectId") Long subjectId);

    //学生端-首页-视频课
    List<StudentHomePage.Video> selectVideo(@Param("appAuditVersion") YesOrNoEnum appAuditVersion, @Param("subjectId") Long subjectId);

    //学生端-首页-最近一堂课
    StudentHomePage.RecentCourses selectRecentCourses(Long studentId);

    //老师端-首页-最近一堂课(趣纠课\VIP课)
    StudentHomePage.RecentCourses selectRecentCoursesPractice(Long teacherId);

    //老师端-首页-最近一堂课(直播课)
    StudentHomePage.RecentCourses selectRecentCoursesLive(Long teacherId);

    /**
     * 本周剩余的课时
     *
     * @param userId 老师id
     * @return
     */
    Integer selectWeekNotStartCourseSchedule(@Param("userId") Long userId);

    /**
     * 本周未布置课后作业数
     *
     * @param userId 老师id
     * @return
     */
    Integer selectHomeworkNotDecorate(@Param("userId") Long userId);

    /**
     * 老师未课后评价课程数
     *
     * @param userId 老师id
     * @return
     */
    Integer selectNotRepliedCourseSchedule(@Param("userId") Long userId);

    /**
     * 老师本周收到的评价
     *
     * @param userId 老师id
     * @return
     */
    Integer selectWeekStudentRepliedCourseSchedule(@Param("userId") Long userId);

    //根据订单号查询开课时间
    List<String> selectStartTime(String orderNo);

    //查询明天有课的老师
    List<SysUser> selectTeacher(String tomorrow);

    //根据老师id统计明日课程
    List<CountVo> selectTypeCount(@Param("teacherId") Long teacherId, @Param("tomorrow") String tomorrow);


    List<TodayNotRepliedAndNotDecorateHomeworkVo> selectTodayNotRepliedAndNotDecorateHomework();

    //统计学员购买课程
    List<RepliedCountVo> countAllReplie(String today);

    //根据id统计评价
    Integer countReplies(Long studentId);

    /**
     * 按年查询首页课程数据
     *
     * @param param
     */
    List<CourseHomeVo.CourseHomeInfoVo> queryCourseHomeOfYear(@Param("param") Map<String, Object> param);

    /**
     * 按月查询首页课程数据
     *
     * @param param
     */
    List<CourseHomeVo.CourseHomeInfoVo> queryCourseHomeOfMonth(@Param("param") Map<String, Object> param);

    //查询老师声部价格
    BigDecimal selectPrice(@Param("teacherId") Long teacherId, @Param("subjectId") Long subjectId, @Param("courseTypeEnum") CourseScheduleEnum courseTypeEnum);

    //查询课程
    List<CourseSchedule> selectSchedule(Integer courseId);

    //更新课程状态
    void updateStartTime(List<CourseSchedule> list);

    //更新课程状态
    void updateEndTime(List<CourseScheduleStudentVo> list);

    //根据日期查课程id
    List<Long> selectIdList(String day);

    //更新老师课酬
    void updateTeacherSalary(List<Long> list);

    //查询已完成课时
    List<CourseCompleteVo> selectComplete();

    //查琴房课列表
    List<PianoClassVo> queryPianoClass(IPage page,@Param("param") MyCourseSearch search);

    //查询待结算课程
    List<UserAccountRecordDto> selectWaitCourse(String day);

    /**
     * 查询课程数
     */
    List<UserBindingTeacherWrapper.CourseNum> selectStudentPianoCourse(@Param("teacherId") Long teacherId, @Param(
            "studentIdList") List<Long> studentIdList);

    /**
     * 查询琴房课
     */
    IPage<UserBindingCourseWrapper> selectBindingUserCoursePage(@Param(
            "page") IPage<UserBindingCourseWrapper> page, @Param("query") TeacherBindingUserQueryInfo.BindingStudentCourseQuery query);

    Integer countStudentOverCourse(@Param("userId") Long userId,
                                @Param("monday") String monday,
                                @Param("sunday") String sunday,
                                @Param("courseStatus") String courseStatus);
}