package com.yonge.cooleshow.biz.dal.service;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.service.IService;
import com.yonge.cooleshow.auth.api.entity.SysUser;
import com.yonge.cooleshow.biz.dal.dto.MusicAlbumSheetDto;
import com.yonge.cooleshow.biz.dal.dto.MusicAlbumSheetSortDto;
import com.yonge.cooleshow.biz.dal.dto.search.MusicAlbumDetailSearch;
import com.yonge.cooleshow.biz.dal.dto.search.MusicAlbumSearch;
import com.yonge.cooleshow.biz.dal.dto.search.StudentMusicAlbumSearch;
import com.yonge.cooleshow.biz.dal.dto.search.StudentMusicSheetOrderSearch;
import com.yonge.cooleshow.biz.dal.entity.MusicAlbum;
import com.yonge.cooleshow.biz.dal.enums.ClientEnum;
import com.yonge.cooleshow.biz.dal.vo.AlbumDetailVo;
import com.yonge.cooleshow.biz.dal.vo.MusicAlbumVo;
import com.yonge.cooleshow.biz.dal.vo.MusicSheetVo;
import com.yonge.cooleshow.biz.dal.vo.ShareProfitVo;
import com.yonge.cooleshow.biz.dal.vo.StudentMusicAlbumOrderVo;
import com.yonge.cooleshow.biz.dal.vo.StudentMusicSheetOrderVo;
import com.yonge.cooleshow.biz.dal.wrapper.TenantAlbumWrapper;
import com.yonge.cooleshow.common.enums.YesOrNoEnum;
import org.apache.ibatis.annotations.Param;

import java.util.List;
import java.util.Map;

/**
 * MusicAlbumService服务类
 * @author yzp
 * @date 2022-03-25 23:46:28
 * @version v1.0
 **/
public interface MusicAlbumService extends IService<MusicAlbum> {

    /**
     * 专辑查询
     *
     * @param page 分页信息
     * @param query 查询条件
     * @return page
     */
    IPage<MusicAlbumVo> selectPage(IPage<MusicAlbumVo> page, MusicAlbumSearch query);

    /**
     * 启用/停用 专辑
     *
     * @param id 专辑id
     * @return
     */
    boolean state(Long id);

    /**
     * 专辑详情信息
     *
     * @param page 分页信息
     * @param query 查询条件
     * @param sysUser
     * @param student
     * @return AlbumDetailVo
     */
    AlbumDetailVo detail(IPage<MusicSheetVo> page, MusicAlbumDetailSearch query, SysUser sysUser, ClientEnum client);

    /**
     * 删除专辑详情关联曲目
     *
     * @param albumId 专辑id
     * @param musicSheetIds 曲目id
     * @return boolean
     */
    boolean delMusicSheet(Long albumId, String musicSheetIds);

    /**
     * 添加专辑详情关联曲目
     *
     * @param albumId 专辑id
     * @param albumMusicList 曲目
     * @param userId 操作用户id
     * @return boolean
     */
    boolean addMusicSheet(Long albumId, List<MusicAlbumSheetDto.AlbumMusic> albumMusicList, Long userId);

    /**
     * 创建专辑
     *
     * @param musicAlbum 专辑对象
     */
    boolean createMusicAlbum(MusicAlbum musicAlbum);

    /**
     * 修改专辑
     *
     * @param musicAlbum 专辑对象
     */
    boolean updateMusicAlbum(MusicAlbum musicAlbum);

    /**
     * 判断是否已收藏
     *
     * @param studentId 学生id
     * @param albumId 专辑id
     * @param clientType ClientEnum
     * @return YesOrNoEnum
     */
    YesOrNoEnum checkFavorite(Long studentId, Long albumId, ClientEnum clientType);

    /**
     * 设置学生收藏/取消收藏 专辑
     *
     * @param studentId 学生id
     * @param albumId 专辑id
     * @param clientType ClientEnum
     * @return boolean
     */
    Boolean setFavorite(Long studentId, Long albumId, ClientEnum clientType);

    /**
     * 收藏专辑列表
     *
     * @param page 分页
     * @param search 查询条件
     * @param clientType ClientEnum
     * @return list
     */
    IPage<MusicAlbumVo> favoriteAlbum(IPage<MusicAlbumVo> page, StudentMusicAlbumSearch search, ClientEnum clientType);

    /**
     * 删除专辑
     *
     * @param id
     * @return
     */
    boolean del(Long id);

    /**
     * 学生查看专辑
     *
     * @param page 分页
     * @param query 查询条件
     * @param clientType ClientEnum
     * @return
     */
    IPage<MusicAlbumVo> selectStudentPage(IPage<MusicAlbumVo> page, MusicAlbumSearch query, ClientEnum clientType);

    /**
     * 修改专辑中 曲目的排序值
     *
     * @param query 排序信息
     * @return
     */
    Boolean updateMusicSheet(MusicAlbumSheetSortDto query);

    /**
     * 专辑分享分润
     */
    ShareProfitVo albumShareProfit(SysUser sysUser, Long albumId);

    IPage<StudentMusicAlbumOrderVo>  selectStudentOrderPage(IPage<StudentMusicAlbumOrderVo> page, StudentMusicSheetOrderSearch query);

    /**
     * 设置专辑收藏的虚拟人数
     */
    void setVirtualNumber();

    TenantAlbumWrapper.TenantAlbum detailAlbum(String albumId);

    TenantAlbumWrapper.TenantAlbum tenantDetailAlbum(String albumId);

    Map<Long, MusicAlbum> getMapByIds(List<Long> bizIds);

    Map<Long,Long> getMusicCountByIds(List<Long> bizIds);
}