package com.yonge.cooleshow.biz.dal.service.impl;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.yonge.cooleshow.biz.dal.dao.CourseScheduleDao;
import com.yonge.cooleshow.biz.dal.dao.StudentStarDao;
import com.yonge.cooleshow.biz.dal.dao.TeacherDao;
import com.yonge.cooleshow.biz.dal.entity.*;
import com.yonge.cooleshow.biz.dal.service.MusicSheetService;
import com.yonge.cooleshow.common.enums.CacheNameEnum;
import com.yonge.cooleshow.biz.dal.service.LiveRoomService;
import org.redisson.api.RBucket;
import org.redisson.api.RedissonClient;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import com.yonge.cooleshow.biz.dal.vo.TeacherTotalVo;
import com.yonge.cooleshow.biz.dal.dto.search.TeacherTotalSearch;
import com.yonge.cooleshow.biz.dal.dao.TeacherTotalDao;
import com.yonge.cooleshow.biz.dal.service.TeacherTotalService;
import org.springframework.util.CollectionUtils;

import java.util.*;
import java.util.function.Consumer;
import java.util.stream.Collectors;


@Service
public class TeacherTotalServiceImpl extends ServiceImpl<TeacherTotalDao, TeacherTotal> implements TeacherTotalService {
    private final static Logger log = LoggerFactory.getLogger(TeacherTotalServiceImpl.class);
    @Autowired
    private TeacherDao teacherDao;
    @Autowired
    private StudentStarDao studentStarDao;
    @Autowired
    private CourseScheduleDao courseScheduleDao;
    @Autowired
    private RedissonClient redissonClient;
    @Autowired
    private LiveRoomService liveRoomService;
    @Autowired
    private MusicSheetService musicSheetService;

    @Override
    public TeacherTotalVo detail(Long id) {
        return baseMapper.detail(id);
    }

    @Override
    public IPage<TeacherTotalVo> selectPage(IPage<TeacherTotalVo> page, TeacherTotalSearch query) {
        return page.setRecords(baseMapper.selectPage(page, query));
    }

    @Override
    public List<TeacherTotal> queryTeacherTotal() {
        List<Teacher> teachers = teacherDao.selectList(Wrappers.<Teacher>emptyWrapper()
                .select("userId"));
        //查询粉丝数
        List<TeacherTotalVo> teacherStarTotal = studentStarDao.queryTeacherTotal();
        Map<Long, TeacherTotalVo> teacherStarMap = new HashMap<>();
        if (!CollectionUtils.isEmpty(teacherStarTotal)) {
            teacherStarMap = teacherStarTotal.stream().collect(Collectors.toMap(TeacherTotalVo::getUserId, o -> o));
        }
        //查询已上未上课时数
        List<TeacherTotalVo> teacherCourseTotal = courseScheduleDao.queryTeacherTotal(null);
        Map<Long, TeacherTotalVo> teacherCourseMap = new HashMap<>();
        if (!CollectionUtils.isEmpty(teacherCourseTotal)) {
            teacherCourseMap = teacherCourseTotal.stream().collect(Collectors.toMap(TeacherTotalVo::getUserId, o -> o));
        }
        //查询老师正在直播的直播间
        List<LiveRoom> templist = liveRoomService.list(Wrappers.<LiveRoom>lambdaQuery()
                .eq(LiveRoom::getLiveState, 1)
                .eq(LiveRoom::getRoomState, 0)
                .eq(LiveRoom::getType, "TEMP"));

        Map<Long, Boolean> liveMap = new HashMap<>();
        for (LiveRoom liveRoom : templist) {
            liveMap.put(liveRoom.getSpeakerId(), true);
        }

        //查询老师曲谱统计
        List<TeacherTotalVo> musicTotals = musicSheetService.queryTeacherTotal(null);
        Map<Long, TeacherTotalVo> teacherMusicMap = new HashMap<>();
        if (!CollectionUtils.isEmpty(musicTotals)) {
            teacherMusicMap = musicTotals.stream().collect(Collectors.toMap(TeacherTotalVo::getUserId, o -> o));
        }

        List<TeacherTotal> resultList = new ArrayList<>();
        for (Teacher teacher : teachers) {
            TeacherTotal teacherTotal = new TeacherTotal();
            teacherTotal.setUserId(teacher.getUserId());
            //粉丝数
            TeacherTotalVo fansTotal = teacherStarMap.get(teacher.getUserId());
            if (null != fansTotal && null != fansTotal.getFansNum()) {
                teacherTotal.setFansNum(fansTotal.getFansNum());
            }

            TeacherTotalVo courseTotal = teacherCourseMap.get(teacher.getUserId());
            if (null != courseTotal) {
                //课时数
                teacherTotal.setExpTime(courseTotal.getExpTime());
                teacherTotal.setUnExpTime(courseTotal.getUnExpTime());
                //星级
                Double starGrade = courseTotal.getStarGrade();
                if (null != starGrade) {
                    Long round = Math.round(starGrade);
                    teacherTotal.setStarGrade(round.doubleValue());
                }
            }
            //是否正在直播
            teacherTotal.setLiveFlag(null == liveMap.get(teacher.getUserId()) ? false : true);

            //曲谱统计
            TeacherTotalVo musicTotal = teacherMusicMap.get(teacher.getUserId());
            if (null != musicTotal) {
                teacherTotal.setMusicAlbumNum(musicTotal.getMusicAlbumNum());
                teacherTotal.setMusicSheetNum(musicTotal.getMusicSheetNum());
            }
            resultList.add(teacherTotal);
            redissonClient.getBucket(CacheNameEnum.TEACHER_TOTAL.getRedisKey(teacherTotal.getUserId()))
                    .set(teacherTotal);
        }
        //统计信息入库
        saveOrUpdateBatch(resultList);
        return resultList;
    }

    int batchSize = 100;

    /***
     * 批量更新插入
     * @author liweifan
     * @param: resultList
     * @updateTime 2022/4/12 16:52
     */
    private void saveOrUpdateBatch(List<TeacherTotal> resultList) {
        List<TeacherTotal> teacherTotals = baseMapper.selectList(Wrappers.<TeacherTotal>emptyWrapper().select("userId"));
        Map<Long, TeacherTotal> teacherTotalMap = new HashMap<>();
        if (!CollectionUtils.isEmpty(teacherTotals)) {
            teacherTotals.forEach(o -> teacherTotalMap.put(o.getUserId(), o));
        }
        //批量插入
        List<TeacherTotal> saveList = resultList.stream()
                .filter(o -> null == teacherTotalMap.get(o.getUserId())).collect(Collectors.toList());
        batchOpera(saveList, this::saveBatch);

        //批量更新
        List<TeacherTotal> updateList = resultList.stream()
                .filter(o -> null != teacherTotalMap.get(o.getUserId())).collect(Collectors.toList());
        batchOpera(updateList, this::updateBatch);
    }

    private void batchOpera(List<TeacherTotal> resultList, Consumer<List<TeacherTotal>> action) {
        int start = 0;
        while (true) {
            int end = start + batchSize;
            List<TeacherTotal> subList;
            if (resultList.size() > end) {
                subList = resultList.subList(start, start + batchSize);
            } else {
                subList = resultList.subList(start, resultList.size());
            }
            //处理sql
            action.accept(subList);
            if (end >= resultList.size()) {
                break;
            }
            start += batchSize;
        }
    }

    private void updateBatch(List<TeacherTotal> resultList) {
        for (TeacherTotal total : resultList) {
            updateById(total);
        }
    }

    @Override
    public Object getRedisValueByKey(String key) {
        return redissonClient.getBucket(key).get();
    }

    @Override
    public TeacherTotal getTotalById(Long id) {
        RBucket<Object> bucket = redissonClient.getBucket(CacheNameEnum.TEACHER_TOTAL.getRedisKey(id));
        if (bucket.isExists()) {
            try {

                return (TeacherTotal) bucket.get();
            }catch (Exception e) {
                log.error("getTotalById error", e);
            }
            //去数据库查询
            TeacherTotal total = totalTeacherTotalById(id);
            if(null != total){
                bucket.set(total);
            }
            return total;
        } else {
            //去数据库查询
            TeacherTotal total = totalTeacherTotalById(id);
            if(null != total){
                bucket.set(total);
            }
            return total;
        }
    }

    @Override
    public void updateTotalCache(TeacherTotal teacherTotal) {
        if (null != teacherTotal && null != teacherTotal.getUserId()) {
            redissonClient.getBucket(CacheNameEnum.TEACHER_TOTAL.getRedisKey(teacherTotal.getUserId())).set(teacherTotal);
        }
    }

    @Override
    public TeacherTotal totalTeacherTotalById(Long id) {
        TeacherTotal teacherTotal = new TeacherTotal();
        teacherTotal.setUserId(id);
        //查询粉丝数
        Integer fansNum = studentStarDao.selectCount(Wrappers.<StudentStar>lambdaQuery()
                .eq(StudentStar::getTeacherId, id)
        );
        teacherTotal.setFansNum(fansNum);

        //查询已上未上课时数
        List<TeacherTotalVo> teacherCourseTotal = courseScheduleDao.queryTeacherTotal(id);
        if (!CollectionUtils.isEmpty(teacherCourseTotal)) {
            TeacherTotalVo courseTotal = teacherCourseTotal.get(0);
            if (null != courseTotal) {
                //课时数
                if (null != courseTotal.getExpTime()) {
                    teacherTotal.setExpTime(courseTotal.getExpTime());
                }
                if (null != courseTotal.getUnExpTime()) {
                    teacherTotal.setUnExpTime(courseTotal.getUnExpTime());
                }
                //星级
                Double starGrade = courseTotal.getStarGrade();
                if (null != starGrade) {
                    Long round = Math.round(starGrade);
                    teacherTotal.setStarGrade(round.doubleValue());
                }
            }
        }
        //查询老师正在直播的直播间
        List<LiveRoom> templist = liveRoomService.list(Wrappers.<LiveRoom>lambdaQuery()
                .eq(LiveRoom::getSpeakerId, id)
                .eq(LiveRoom::getLiveState, 1)
                .eq(LiveRoom::getRoomState, 0)
                .eq(LiveRoom::getType, "TEMP"));

        if (CollectionUtils.isEmpty(templist)) {
            teacherTotal.setLiveFlag(false);
        } else {
            teacherTotal.setLiveFlag(true);
        }

        //查询老师曲谱数
        List<TeacherTotalVo> musicTotals = musicSheetService.queryTeacherTotal(id);
        if (!CollectionUtils.isEmpty(musicTotals)) {
            TeacherTotalVo musicTotal = musicTotals.get(0);
            if (null != musicTotal) {
                teacherTotal.setMusicAlbumNum(musicTotal.getMusicAlbumNum());
                teacherTotal.setMusicSheetNum(musicTotal.getMusicSheetNum());
            }
        }

        redissonClient.getBucket(CacheNameEnum.TEACHER_TOTAL.getRedisKey(teacherTotal.getUserId()))
                .set(teacherTotal);

        TeacherTotal old = getById(id);
        if (null != old) {
            updateById(teacherTotal);
        } else {
            baseMapper.save(teacherTotal);
        }
        return teacherTotal;
    }
}