package com.yonge.cooleshow.biz.dal.wrapper;

import com.alibaba.fastjson.JSON;
import com.microsvc.toolkit.common.response.paging.QueryInfo;
import io.swagger.annotations.ApiModel;
import io.swagger.annotations.ApiModelProperty;
import java.util.Optional;

import lombok.AllArgsConstructor;
import lombok.Builder;
import lombok.Data;
import lombok.NoArgsConstructor;
import org.apache.commons.lang3.StringUtils;

/**
 * 微信配置信息
 * 2023-07-21 17:32:49
 */
@ApiModel(value = "WxConfigInfoWrapper对象", description = "微信配置信息查询对象")
public class WxConfigInfoWrapper {

    @Data
	@Builder
    @NoArgsConstructor
    @AllArgsConstructor
    @ApiModel(" WxConfigInfoQuery-微信配置信息")
    public static class WxConfigInfoQuery implements QueryInfo {
    
    	@ApiModelProperty("当前页")
        private Integer page;
        
        @ApiModelProperty("分页行数")
        private Integer rows;
        
        @ApiModelProperty("关键字匹配")
		private String keyword;
        
        public String getKeyword() {
            return Optional.ofNullable(keyword).filter(StringUtils::isNotBlank).orElse(null);
        }
        
        public String jsonString() {
            return JSON.toJSONString(this);
        }

        public static WxConfigInfoQuery from(String json) {
            return JSON.parseObject(json, WxConfigInfoQuery.class);
        }
    }  

	@ApiModel(" WxConfigInfo-微信配置信息")
    public static class WxConfigInfo {
        
        public String jsonString() {
            return JSON.toJSONString(this);
        }

        public static WxConfigInfo from(String json) {
            return JSON.parseObject(json, WxConfigInfo.class);
        }
	}

}